
from lxml.builder import ElementMaker

from moai.metadata.mods import NL_MODS, XSI_NS

        
class DIDL(object):
    """A metadata prefix implementing the DARE DIDL metadata format
    this format is registered under the name "didl"
    Note that this format re-uses oai_dc and mods formats that come with
    MOAI by default
    """
        
    def __init__(self, prefix, config, db):
        self.prefix = prefix
        self.config = config
        self.db = db

        self.ns = {'didl': "urn:mpeg:mpeg21:2002:02-DIDL-NS",
                   'dii': "urn:mpeg:mpeg21:2002:01-DII-NS",
                   'dip': "urn:mpeg:mpeg21:2005:01-DIP-NS",
                   'dcterms': "http://purl.org/dc/terms/",
                   'xsi': "http://www.w3.org/2001/XMLSchema-instance",
                   'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
                   'dc': 'http://purl.org/dc/elements/1.1/',
                   }

        self.schemas = {'didl':'http://standards.iso.org/ittf/PubliclyAvailableStandards/MPEG-21_schema_files/did/didl.xsd',
                        'dii': 'http://standards.iso.org/ittf/PubliclyAvailableStandards/MPEG-21_schema_files/dii/dii.xsd',
                        'dip': 'http://standards.iso.org/ittf/PubliclyAvailableStandards/MPEG-21_schema_files/dip/dip.xsd'}
        
    def get_namespace(self):
        return self.ns[self.prefix]

    def get_schema_location(self):
        return self.schemas[self.prefix]

        
    def __call__(self, element, metadata):
        data = metadata.record
        
        DIDL = ElementMaker(namespace=self.ns['didl'], nsmap=self.ns)
        DII = ElementMaker(namespace=self.ns['dii'])
        DIP = ElementMaker(namespace=self.ns['dip'])
        RDF = ElementMaker(namespace=self.ns['rdf'])
        DCTERMS = ElementMaker(namespace=self.ns['dcterms'])

        oai_url = (self.config.url+'?verb=GetRecord&'
                   'metadataPrefix=%s&identifier=%s' % (
            self.prefix,
            data['id']))

        id_url = data['metadata'].get('url', [None])[0]        

        # generate mods for this feed
        mods_data = DIDL.Resource(mimeType="application/xml")
        NL_MODS('mods', self.config, self.db)(mods_data, metadata)

        asset_data = []

        descriptive_metadata = RDF.type()
        descriptive_metadata.attrib['{%s}resource' % self.ns['rdf']] = (
            'info:eu-repo/semantics/descriptiveMetadata')
        
        didl = DIDL.DIDL(
            DIDL.Item(
             DIDL.Descriptor(
              DIDL.Statement(
               DCTERMS.modified(data['modified'].isoformat().split('.')[0]),
               mimeType="application/xml"
               )
              ),
             DIDL.Component(
              DIDL.Resource(ref=id_url or oai_url,mimeType="application/xml")
              ),
             DIDL.Item(
              DIDL.Descriptor(
               DIDL.Statement(descriptive_metadata, mimeType="application/xml")
               ),
              DIDL.Component(
                DIDL.Descriptor(
                   DIDL.Statement("mods", mimeType="text/plain")),
                mods_data)
              ),
             )
            )

        object_file = RDF.type()
        object_file.attrib['{%s}resource' % self.ns['rdf']] = (
            'info:eu-repo/semantics/objectFile')
        for root_item in didl:
            for asset in data['metadata'].get('asset', []):
                url = asset['url']
                if not url.startswith('http://'):
                    url = self.config.url.rstrip('/') + '/' + url.lstrip('/')
                item = DIDL.Item(
                    DIDL.Descriptor(
                     DIDL.Statement(object_file, mimeType="application/xml")
                     )
                    )
                access = asset.get('access')
                if access == 'open':
                    access = (
                        'http://purl.org/eprint/accessRights/OpenAccess')
                elif access == 'restricted':
                    access = (
                        'http://purl.org/eprint/accessRights/RestrictedAccess')
                elif access == 'closed':
                    access = (
                        'http://purl.org/eprint/accessRights/ClosedAccess')
                if access:
                    item.append(
                        DIDL.Descriptor(
                        DIDL.Statement(DCTERMS.accessRights(access),
                                       mimeType="application/xml")))
                for modified in asset.get('modified', []):
                    item.append(
                        DIDL.Descriptor(
                                DIDL.Statement(DCTERMS.modified(modified),
                                               mimeType="application/xml")))
                    
                item.append(
                    DIDL.Component(
                     DIDL.Resource(mimeType=asset['mimetype'],
                                   ref=url)
                     )
                    )

                root_item.append(item)
            break
        
        human_start_page = RDF.type()
        human_start_page.attrib['{%s}resource' % self.ns['rdf']] = (
            'info:eu-repo/semantics/humanStartPage')
        if data['metadata'].get('url'):
             item = DIDL.Item(
                 DIDL.Descriptor(
                  DIDL.Statement(human_start_page, mimeType="application/xml")
                  ),
                 DIDL.Component(
                  DIDL.Resource(mimeType="text/html", ref=data['metadata']['url'][0])
                 )
                )
             root_item.append(item)
        
        didl.attrib['{%s}schemaLocation' % XSI_NS] = (
            '%s %s %s %s %s %s' % (self.ns['didl'],
                                   self.schemas['didl'],
                                   self.ns['dii'],
                                   self.schemas['dii'],
                                   self.ns['dip'],
                                   self.schemas['dip']))
        element.append(didl)




from lxml.builder import ElementMaker

XSI_NS = 'http://www.w3.org/2001/XMLSchema-instance'
  
class OAIDC(object):
    """The standard OAI Dublin Core metadata format.
    
    Every OAI feed should at least provide this format.

    It is registered under the name 'oai_dc'
    """
    
    def __init__(self, prefix, config, db):
        self.prefix = prefix
        self.config = config
        self.db = db

        self.ns = {'oai_dc': 'http://www.openarchives.org/OAI/2.0/oai_dc/',
                   'dc':'http://purl.org/dc/elements/1.1/'}
        self.schemas = {'oai_dc': 'http://www.openarchives.org/OAI/2.0/oai_dc.xsd'}
        
    def get_namespace(self):
        return self.ns[self.prefix]

    def get_schema_location(self):
        return self.schemas[self.prefix]
    
    def __call__(self, element, metadata):

        data = metadata.record
        
        OAI_DC =  ElementMaker(namespace=self.ns['oai_dc'],
                               nsmap =self.ns)
        DC = ElementMaker(namespace=self.ns['dc'])

        oai_dc = OAI_DC.dc()
        oai_dc.attrib['{%s}schemaLocation' % XSI_NS] = '%s %s' % (
            self.ns['oai_dc'],
            self.schemas['oai_dc'])

        for field in ['title', 'creator', 'subject', 'description',
                      'publisher', 'contributor', 'type', 'format',
                      'identifier', 'source', 'language', 'date',
                      'relation', 'coverage', 'rights']:
            el = getattr(DC, field)
            for value in data['metadata'].get(field, []):
                if field == 'identifier' and data['metadata'].get('url'):
                    value = data['metadata']['url'][0]
                oai_dc.append(el(value))
        
        element.append(oai_dc)

class MODS(object):
    """This is the minimods formats as defined by DARE.

    It is registered as prefix 'mods'.'
    """
    
    def __init__(self, prefix, config, db):
        self.prefix = prefix
        self.config = config
        self.db = db

        self.ns = {'mods': 'http://www.loc.gov/mods/v3',
                   'xml':'http://www.w3.org/XML/1998/namespace',
                   'dai': 'info:eu-repo/dai'}

        self.schemas = {
           'mods': 'http://www.loc.gov/standards/mods/v3/mods-3-2.xsd'}
        
        
    def get_namespace(self):
        return self.ns[self.prefix]

    def get_schema_location(self):
        return self.schemas[self.prefix]
    
    def __call__(self, element, metadata):

        data = metadata.record
        MODS = ElementMaker(namespace=self.ns['mods'], nsmap=self.ns)
        DAI = ElementMaker(namespace=self.ns['dai'], nsmap=self.ns)
        mods = MODS.mods(version="3.2")

        if data['metadata'].get('identifier'):
            mods.append(MODS.identifier(data['metadata']['identifier'][0],
                                        type="uri"))

        if data['metadata'].get('url'):
            mods.append(MODS.location(MODS.url(data['metadata']['url'][0])))

        if data['metadata'].get('title'):
            titleInfo = MODS.titleInfo(
                MODS.title(data['metadata']['title'][0])
                )
            titleInfo.attrib['{%s}lang' % self.ns['xml']] = data['metadata'].get(
                'language', ['en'])[0]
            mods.append(titleInfo)
            
        if data['metadata'].get('description'):
            mods.append(MODS.abstract(data['metadata']['description'][0]))


        for ctype in ['author', 'editor', 'advisor']:
            contributor_data = []
            for id in data['metadata'].get('%s_rel' % ctype, []):
                contributor = self.db.get_metadata(id)
                contributor['id'] = id
                contributor_data.append(contributor)

            if data['metadata'].get('%s_data' % ctype):
                contributor_data = [s for s in data['metadata'][
                    '%s_data' % ctype]]
        
            if not contributor_data:            
                contributor_data = [{'name':[a]} for a in data[
                    'metadata'].get(ctype, [])]

            dai_list = []
            for contributor in contributor_data:
                unique_id = data['record']['id'] + '_' + contributor.get(
                    'id', contributor['name'][0])
                if unique_id[0].isdigit():
                    unique_id = '_'+unique_id
                unique_id = unique_id.replace(':', '')
                name = MODS.name(
                    MODS.displayForm(contributor['name'][0]),
                    type='personal',
                    ID=unique_id
                    )
                surname = contributor.get('surname')
                if surname:
                    name.append(MODS.namePart(surname[0], type="family"))
                firstname = contributor.get('firstname')
                if firstname:
                    name.append(MODS.namePart(firstname[0], type="given"))

                role = contributor.get('role')
                if role:
                    role = role[0]
                else:
                    roles = {'author': 'aut', 'editor': 'edt', 'advisor':'ths'}
                    role = roles[ctype]
                name.append(                    
                    MODS.role(
                    MODS.roleTerm(role,
                                  type='code',
                                  authority='marcrelator')
                    ))
                mods.append(name)
                dai = contributor.get('dai')
                if dai:
                    dai_list.append((unique_id, dai))
            if dai_list:
                daiList = DAI.daiList()
                for id, dai in dai_list:
                    daiList.append(DAI.identifier(
                        dai[0],
                        IDref=id,
                        authority='info:eu-repo/dai/nl'))
                
                mods.append(MODS.extension(daiList))


        dgg = data['metadata'].get('degree_grantor')
        if dgg:
            mods.append(MODS.name(
                MODS.namePart(dgg[0]),
                MODS.role(
                  MODS.roleTerm('dgg',
                                authority="marcrelator",
                                type="code")
                ),
                type="corporate"))

        if data['metadata'].get('language'):
            mods.append(MODS.language(
                MODS.languageTerm(data['metadata']['language'][0],
                                  type="code",
                                  authority="rfc3066")))

        for host in ['journal', 'series']:
            title = data['metadata'].get('%s_title' % host)
            part_type = {'journal': 'host'}.get(host, host)
            relitem = MODS.relatedItem(type=part_type)
            if title:
                relitem.append(MODS.titleInfo(MODS.title(title[0])))
            else:
                continue
            issn = data['metadata'].get('%s_issn' % host)
            if issn:
                relitem.append(
                    MODS.identifier('urn:issn:%s' % issn[0],
                                    type="uri"))
            volume = data['metadata'].get('%s_volume' % host)
            issue = data['metadata'].get('%s_issue' % host)
            start_page = data['metadata'].get('%s_start_page' % host)
            end_page = data['metadata'].get('%s_end_page' % host)
            if volume or issue or end_page or start_page:
                part = MODS.part()
                if volume:
                    part.append(MODS.detail(MODS.number(volume[0]),
                                            type="volume"))
                if issue:
                    part.append(MODS.detail(MODS.number(issue[0]),
                                            type="issue"))
                if start_page or end_page:
                    extent = MODS.extent(unit="page")
                    if start_page:
                        extent.append(MODS.start(start_page[0]))
                    if end_page:
                        extent.append(MODS.end(end_page[0]))
                    part.append(extent)
                relitem.append(part)
            if data['metadata'].get('%s_publisher' % host):
                relitem.append(
                    MODS.originInfo(
                      MODS.publisher(
                        data['metadata']['%s_publisher' % host][0])))
                
            mods.append(relitem)

        origin = MODS.originInfo()
        mods.append(origin)
        if data['metadata'].get('publisher'):
            origin.append(MODS.publisher(data['metadata']['publisher'][0]))
        if data['metadata'].get('date'):
            origin.append(MODS.dateIssued(data['metadata']['date'][0],
                                        encoding='iso8601'))

        mods.append(MODS.typeOfResource('text'))        
        if data['metadata'].get('dare_type'):
            mods.append(MODS.genre(data['metadata']['dare_type'][0]))

        
        classifications = data['metadata'].get('classification', [])
        for classification in classifications:
            if classification.count('#') == 1:
                authority, value = classification.split('#')
                mods.append(MODS.classification(value, authority=authority))
            else:
                mods.append(MODS.classification(classification))
        
        subjects = data['metadata'].get('subject', [])
        if subjects:
            s_el = MODS.subject()
            for subject in subjects:
                s_el.append(MODS.topic(subject))
            mods.append(s_el)

        if data['metadata'].get('rights'):
            mods.append(MODS.accessCondition(data['metadata']['rights'][0]))
        
            
        mods.attrib['{%s}schemaLocation' % XSI_NS] = '%s %s' % (
            self.ns['mods'],
            self.schemas['mods'])
        
        element.append(mods)




import sys
import os
import time
import datetime
import pkg_resources
from pkg_resources import iter_entry_points
import ConfigParser

from optparse import OptionParser

from moai.utils import (get_duration,
                        get_moai_log,
                        ProgressBar)
from moai.database import SQLDatabase

VERSION = pkg_resources.working_set.by_key['moai'].version
                 
def update_moai():
    usage = "usage: %prog [options] profilename"
    version = "%%prog %s" % VERSION

    parser = OptionParser(usage, version=version)

    parser.add_option("-v", "--verbose", dest="verbose",
                      help="print logging at info level",
                      action="store_true")
    parser.add_option('-d', '--debug', dest='debug',
                      help="print traceback and quit on error",
                      action='store_true')
    parser.add_option("-q", "--quiet", dest="quiet",
                      help="be quiet, do not output and info",
                      action="store_true")
    parser.add_option("", "--config", dest="config",
                      help="specify settings file",
                      action="store")
    parser.add_option("", "--date", dest="from_date",
                      help="Only update database from a specific date",
                      action="store")
        
    options, args = parser.parse_args()
    if not len(args):
        profile_name = 'default'
    else:
        profile_name = args[0]
    if options.config:
        config_path = options.config
    else:
        config_path = os.path.join(
            os.path.dirname(os.path.abspath(os.path.dirname(sys.argv[0]))),
            'settings.ini')
    if not os.path.isfile(config_path):
        sys.stderr.write('No config_path file found at %s,\n'
                         'Start script from other directory '
                         'or use --config option\n''' % config_path)
        sys.exit(1)
    configfile = ConfigParser.ConfigParser()
    configfile.read(config_path)
    profiles = []
    config = {}
    for section in configfile.sections():
        if not configfile.has_option(section, 'use'):
            continue
        if configfile.get(section, 'use') == 'egg:moai':
            profiles.append(section.split(':', 1)[1])
        if profile_name == section.split(':', 1)[1]:
            for option in configfile.options(section):
                config[option] = configfile.get(section, option)
            
    if not profile_name in profiles:
        if profile_name == 'default':
            sys.stderr.write(
                'No profile name specified, use --help for more info\n')
        else:
            sys.stderr.write('unknown profile: %s\n' % profile_name)
        sys.stderr.write('(known profiles are: %s)\n' % ', '.join(profiles))
        sys.exit(1)

    if options.from_date:
        if 'T' in options.from_date:
            fmt = '%Y-%m-%dT%H:%M:%S'
        else:
            fmt = '%Y-%m-%d'
        from_date = datetime.datetime(
            *time.strptime(options.from_date, fmt)[:6])
    else:
        from_date = None

    database = SQLDatabase(config['database'])

    ContentClass = None
    for content_point in iter_entry_points(group='moai.content',
                                           name=config['content']):
        ContentClass = content_point.load()

    if ContentClass is None:
        sys.stderr.write('Unknown content class: %s\n' % (config['content'],))
        sys.exit(1)

    provider_name = config['provider'].split(':', 1)[0]
    provider = None
    for provider_point in iter_entry_points(group='moai.provider',
                                           name=provider_name):
        provider = provider_point.load()(config['provider'])

    if provider is None:
        sys.stderr.write('Unknown provider: %s\n' % (provider_name,))
        sys.exit(1)

    log = get_moai_log()
    provider.set_logger(log)

    progress = ProgressBar()
    starttime = time.time()

    sys.stderr.write('Updating content provider..')
    count = 0    
    for id in provider.update(from_date):
        if not options.quiet and not options.verbose:
            progress.animate('Updating content provider: %s' % id)
            count += 1

    if not options.quiet and not options.verbose:
        progress.write('')
        print
        print >> sys.stderr, ('Content provider returned %s '
                              'new/modified objects' % count)
        print >> sys.stderr
    
    total = provider.count()

    count = 0
    ignore_count = 0
    error_count = 0
    flush_threshold = int(config.get('forcedflush', '10000'))
    for content_id in provider.get_content_ids():
        count += 1
        try:
            raw_data = provider.get_content_by_id(content_id)
        except Exception, err:
            if options.debug:
                raise
            log.error('Error retrieving data %s from provider: %s' % (
                content_id, str(err)))
            error_count += 1
            progress.tick(count, total)
            continue

        try:
            content = ContentClass(provider)
            success = content.update(raw_data)
        except Exception, err:
            if options.debug:
                raise
            log.error('Error converting data %s to content: %s' % (
                content_id, str(err)))
            error_count += 1
            progress.tick(count, total)
            continue
        
        if success is False:
            log.warning('Ignoring %s' % content_id)
            ignore_count += 1
            progress.tick(count, total)
            continue
        
        try:
            database.update_record(content.id,
                                   content.modified,
                                   content.deleted,
                                   content.sets,
                                   content.metadata)
        except Exception, err:
            if options.debug:
                raise
            log.error('Error inserting %s into database: %s' % (
                content.id, str(err)))
            error_count += 1
            progress.tick(count, total)
            continue
            
        if count % flush_threshold == 0:
            log.info('Flushing database')
            database.flush()
        progress.tick(count, total)
        
    log.info('Flushing database')
    database.flush()
    duration = get_duration(starttime)
    print >> sys.stderr, ''
    msg = 'Updating database with %s objects took %s' % (total, duration)
    log.info(msg)
    if not options.verbose and not options.quiet:
        print >> sys.stderr, msg

    if error_count:
        msg = '%s error%s occurred during updating' % (
            error_count,
            {1: ''}.get(error_count, 's'))
        log.warning(msg)
        if not options.verbose and not options.quiet:
            print >> sys.stderr, msg




import os
import time
import fnmatch


class FileBasedContentProvider(object):
    """Provides content by reading directories of files
    Implements the :ref:`IContentProvider` interface
    """
    def __init__(self, uri, content_filter="*"):
        assert uri.startswith('file://'), 'unknown uri format'
        path = uri[7:]
        basedir, filename = os.path.split(path)
        if '*' in filename or '?' in filename:
            content_filter = filename
            self._path = basedir
        else:
            self._path = path
        self._filter = content_filter
        self._content = {}

    def set_logger(self, log):
        self._log = log
        
    def _harvest(self, from_time=None):
        result = {}
        for p, d, f in os.walk(self._path):
            for directory in d:
                if directory.startswith('.'):
                    d.remove(directory)
            for file in f:
                if file[0] in ['.', '#']:
                    continue
                if not fnmatch.fnmatch(file, self._filter):
                    continue
                path = os.path.join(p, file)
                if not from_time is None:
                    mtime= os.path.getmtime(path)
                    if mtime < from_time:
                        continue
                id = os.path.basename(path)
                result[id] = path
        return result

    def update(self, from_date=None):
        if from_date is None:
            from_time = None
        else:
            from_time = time.mktime(from_date.timetuple())
        result = self._harvest(from_time=from_time)
        self._content.update(result)
        return result.keys()

    def count(self):
        return len(self._content)

    def get_content_ids(self):
        for id in self._content.keys():
            yield id

    def get_content_by_id(self, id):
        return self._content[id]



import os
import urllib2
import hashlib

from lxml import etree

from moai.provider.oai import OAIBasedContentProvider

class FOXMLFile(object):

    def __init__(self, file_obj):
        self._doc = etree.parse(file_obj)
        self._ns = 'info:fedora/fedora-system:def/foxml#'

    def get_property(self, name):
        properties = self._doc.xpath(
            '//foxml:property[@NAME="%s"]' % name,
            namespaces={'foxml':self._ns})
        if not properties: return
        value = properties[-1].get('VALUE')
        if value: return value.decode('utf8')
        
    def get_xml_ids(self):
        ids = self._doc.xpath(
            '//foxml:datastream[@CONTROL_GROUP="X"]/@ID',
            namespaces={'foxml':self._ns})
        return [i.decode('utf8') for i in ids]
    def get_ids(self):
        ids = self._doc.xpath(
            '//foxml:datastream/@ID',
            namespaces={'foxml':self._ns})
        return [i.decode('utf8') for i in ids]

    def get_xml(self, id):
        contents = self._doc.xpath(
            ('//foxml:datastream[@CONTROL_GROUP="X" and '
             '@ID="%s"]/foxml:datastreamVersion/foxml:xmlContent' % id),
            namespaces={'foxml':self._ns})
        if not contents:
            return
        for child in contents[-1]:
            xml = etree.tostring(child, encoding='UTF8', pretty_print=True)
            break
        xml = xml.strip()
        if not isinstance(xml, unicode):
            xml = xml.decode('utf8')
        return xml

    def get_location(self, id):
        locations = self._doc.xpath(
            ('//foxml:datastream['
             '@ID="%s"]/foxml:datastreamVersion/'
             'foxml:contentLocation[@TYPE="URL"]/@REF' % id),
            namespaces={'foxml':self._ns})
        if not locations:
            return
        return locations[-1].decode('utf8')
    
    def get_digest(self, id):
        digests = self._doc.xpath(
            ('//foxml:datastream['
             '@ID="%s"]/foxml:datastreamVersion/'
             'foxml:contentDigest[@TYPE="MD5"]/@DIGEST' % id),
            namespaces={'foxml':self._ns})
        if not digests:
            return
        return digests[-1].decode('utf8')
    
    def get_mimetype(self, id):
        mimes = self._doc.xpath(
            ('//foxml:datastream['
             '@ID="%s"]/foxml:datastreamVersion/@MIMETYPE' % id),
            namespaces={'foxml':self._ns})
        if not mimes:
            return
        return mimes[-1].decode('utf8')
        
    def get_label(self, id):
        labels = self._doc.xpath(
            ('//foxml:datastream['
             '@ID="%s"]/foxml:datastreamVersion/@LABEL' % id),
            namespaces={'foxml':self._ns})
        if not labels:
            return
        label = labels[-1]
        if not isinstance(label, unicode):
            label = label.decode('utf8')
        return label
        

class FedoraBasedContentProvider(OAIBasedContentProvider):
    """Providers content by harvesting a Fedora Commons OAI feed.
    Then uses the content from a specific datastream, or retrieves the
    full foxml file if no datastream is provided
    Implements the :ref:`IContentProvider` interface
    """
    
    def __init__(self, fedora_url, output_path,
                 datastream_name=None, username=None, password=None):
        oai_url = '%s/oai' % fedora_url
        super(FedoraBasedContentProvider, self).__init__(oai_url, output_path)
        self._stream = datastream_name
        self._fedora_url = fedora_url
        self._user = username
        self._pass = password
        if not os.path.isdir(output_path):
            os.mkdir(output_path)

    def _get_id(self, header):
        fedora_id = ':'.join(header.identifier().split(':')[2:])
        return fedora_id

    def _process_record(self, header, element):

        fedora_id = self._get_id(header)
        
        if self._stream is None:
            url = '%s/objects/%s/objectXML' % (self._fedora_url,
                                               fedora_id)
        else:
            # get only a specific data stream
            url = '%s/get/%s/%s' % (self._fedora_url,
                                    fedora_id, 
                                    self._stream)

        if self._user and self._pass:
            password = ('%s:%s' % (self._user, self._pass)).strip().encode('base64')
            headers = {'Authorization': 'Basic %s' % password}
            request = urllib2.Request(url, headers=headers)
        else:
            request = urllib2.Request(url)

        try:
            fp = urllib2.urlopen(request)
            xml_data = fp.read()
            fp.close()
        except urllib2.HTTPError, err:
            self._log.warning('HTTP %s -> Can not get Fedora data: %s' % (err.code, url))
            return False

        directory = hashlib.md5(fedora_id).hexdigest()[:3]

        path = os.path.join(self._path, directory)
        if not os.path.isdir(path):
            os.mkdir(path)
                
        path = os.path.join(path, '%s.xml' % fedora_id)
        
        fp = open(path, 'w')
        fp.write(xml_data)
        fp.close()
        return True



# coding=utf8
import os
from unittest import TestCase, TestSuite, makeSuite
import doctest
import datetime

try:
    import urllib2
    urlopen = urllib2.urlopen
except ImportError:
    import urllib
    from urllib import request
    urlopen = request.urlopen

from lxml import etree
import wsgi_intercept
try:
    from wsgi_intercept.urllib2_intercept import install_opener
except ImportError:
    from wsgi_intercept.urllib_intercept import install_opener

from moai.utils import XPath
from moai.database import SQLDatabase as Database
from moai.server import Server, FeedConfig
from moai.wsgi import MOAIWSGIApp
from moai.provider.file import FileBasedContentProvider
from moai.example import ExampleContent
install_opener()

FLAGS = doctest.NORMALIZE_WHITESPACE + doctest.ELLIPSIS
GLOBS = {}

class XPathUtilTest(TestCase):

    def test_string(self):
        doc = etree.fromstring(
            '''<doc>
                 <string>test</string>
                 <string/>
                 <string>   test     </string>
                 <string>test<foo/>more test</string>
                 <string>tëst</string>
               </doc>''')
        xpath = XPath(doc)
        self.assertEquals(xpath.strings('/doc/string'),
                          [u'test', u'test', u'test', u'tëst'])
    def test_boolean(self):
        doc = etree.fromstring(
            '''<doc>
                 <bool>yes</bool>
                 <bool>true</bool>
                 <bool>False</bool>
                 <bool>NO</bool>
               </doc>''')
        xpath = XPath(doc)
        self.assertEquals(xpath.booleans('/doc/bool'),
                          [True, True, False, False])

    def test_number(self):
        doc = etree.fromstring(
            '''<doc>
                 <number>1</number>
                 <number>3.33333333333</number>
                 <number>-75</number>
                 <number>-0.75</number>
               </doc>''')
        xpath = XPath(doc)
        numbers = xpath.numbers('/doc/number')
        self.assertEquals(numbers,
                          [1, 3.33333333333, -75, -0.75])
        self.assertEquals([type(i) for i in numbers],
                          [int, float, int, float])

        
    def test_date(self):
        doc = etree.fromstring(
            '''<doc>
                 <date>2010-01-04</date>
                 <date>2010/01/04</date>
                 <date>2010-01-04T12:43:33Z</date>
                 <date>2010-01-04T12:43:33</date>
               </doc>''')
        xpath = XPath(doc)
        self.assertEquals(xpath.dates('/doc/date'),
                          [datetime.datetime(2010, 1, 4, 0, 0),
                           datetime.datetime(2010, 1, 4, 0, 0),
                           datetime.datetime(2010, 1, 4, 12, 43, 33),
                           datetime.datetime(2010, 1, 4, 12, 43, 33)])

    def test_tags(self):
        doc = etree.fromstring('<doc><a/><b/><s:c xmlns:s="urn:spam"/></doc>')
        xpath = XPath(doc)
        self.assertEquals(xpath.tags('/doc/*'), [u'a', u'b', u'c'])

    def test_namespaces(self):
        doc = etree.fromstring(
            '<doc xmlns="urn:spam"><string>Spam!</string></doc>')
        xpath = XPath(doc, nsmap={'spam': 'urn:spam'})
        self.assertEquals(xpath.string('//spam:string'), u'Spam!')

class DatabaseTest(TestCase):
    def setUp(self):
        self.db = Database()
        
    def tearDown(self):
        del self.db
        
    def test_update(self):
        # db is empty
        self.assertEquals(self.db.record_count(), 0)
        # let's add a record
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False,
                              {},
                              {u'title': u'Spam!'})
        self.db.flush()
        self.assertEquals(self.db.record_count(), 1)
        # check if all values are there
        record = self.db.get_record(u'oai:spam')
        self.assertEquals(record['id'], u'oai:spam')
        self.assertEquals(record['deleted'], False)
        self.assertEquals(record['modified'],
                          datetime.datetime(2010, 10, 13, 12, 30, 0))
        self.assertEquals(record['sets'], [])
        self.assertEquals(record['metadata'], {u'title': u'Spam!'})
        # change a metadata value
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 10, 13, 12, 30, 1),
                              False,
                              {},
                              {u'title': u'Ham!'})
        self.db.flush()
        self.assertEquals(self.db.record_count(), 1)
        # check if metadata was changed
        record = self.db.get_record(u'oai:spam')
        self.assertEquals(record['metadata'], {u'title': u'Ham!'})
        # remove the record
        self.db.remove_record(u'oai:spam')
        self.assertEquals(self.db.record_count(), 0)
        
    def test_setrefs(self):
        # add a record that references a set
        self.assertEquals(self.db.set_count(), 0)
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False,
                              {u'spamset': {u'name': u'Spam Set',
                                            u'description': u'spam spam spam',
                                            u'hidden': False}},
                              {u'title': u'Spam!'})
        self.db.flush()
        self.assertEquals(self.db.record_count(), 1)
        self.assertEquals(self.db.set_count(), 1)
        # check if all values are there
        record = self.db.get_record(u'oai:spam')
        self.assertEquals(record['sets'], [u'spamset'])
        set = self.db.get_set(u'spamset')
        self.assertEquals(set['id'], u'spamset')
        self.assertEquals(set['name'], u'Spam Set')
        self.assertEquals(set['description'], u'spam spam spam')
        self.assertEquals(set['hidden'], False)
        # now, we'll change the record to use the hamset
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False,
                              {u'hamset': {u'name': u'Ham Set',
                                            u'description': u'ham ham ham',
                                            u'hidden': False}},
                              {u'title': u'Ham!'})
        self.db.flush()
        self.assertEquals(self.db.record_count(), 1)
        # note that we now have 2 sets, the spam set is not removed
        self.assertEquals(self.db.set_count(), 2)
        # however, the spam record only has one reference 
        record = self.db.get_record(u'oai:spam')
        self.assertEquals(record['sets'], [u'hamset'])
        # if the set is removed then all references to that set are
        # also removed
        self.db.remove_set(u'hamset')
        record = self.db.get_record(u'oai:spam')
        self.assertEquals(record['sets'], [])
        self.assertEquals(self.db.set_count(), 1)

    def test_hidden_sets(self):
        # hidden sets are not added to the record setrefs list,
        # they are there though, for filtering purposes
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False,
                              {u'spamset': {u'name': u'Spam Set',
                                            u'description': u'spam spam spam',
                                            u'hidden': False},
                               u'hamset': {u'name': u'Ham Set',
                                           u'description': u'ham ham ham',
                                           u'hidden': True}},
                              {u'title': u'Spam!'})
        self.db.flush()
        self.assertEquals(self.db.get_setrefs(u'oai:spam'), [u'spamset'])
        self.assertEquals(self.db.get_setrefs(u'oai:spam',
                                              include_hidden_sets=True),
                          [u'hamset', u'spamset'])
        # hidden sets are also never shown in the oai sets listing
        self.assertEquals(list(self.db.oai_sets()),
                          [{'description': u'spam spam spam',
                            'id': u'spamset',
                            'name': u'Spam Set'}] )

    def test_earliest_datestamp(self):
        self.assertEquals(self.db.oai_earliest_datestamp(),
                          datetime.datetime(1970, 1, 1, 0, 0))
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2009, 10, 13, 12, 30, 0),
                              False, {}, {})
        self.db.update_record(u'oai:ham',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False, {}, {})
        self.db.flush()
        self.assertEquals(self.db.oai_earliest_datestamp(),
                          datetime.datetime(2009, 10, 13, 12, 30))

    def test_oai_query_dates(self):
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 1, 1, 0, 0, 0),
                              False, {u'spamset':{u'name':u'spam'}},
                              {})
        self.db.update_record(u'oai:ham',
                              datetime.datetime(2009, 1, 1, 0, 0, 0),
                              False, {u'hamset':{u'name':u'ham'}},
                              {})
        self.db.flush()
        self.assertEquals(
            list(self.db.oai_query()),
            [{'deleted': False,
              'sets': [u'spamset'],
              'metadata': {},
              'id': u'oai:spam',
              'modified': datetime.datetime(2010, 1, 1, 0, 0)},
             {'deleted': False,
              'sets': [u'hamset'],
              'metadata': {},
              'id': u'oai:ham',
              'modified': datetime.datetime(2009, 1, 1, 0, 0)}])
        # date slices
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(
            from_date=datetime.datetime(2009, 6, 1, 0, 0))],
            [u'oai:spam'])
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(
            until_date=datetime.datetime(2009, 6, 1, 0, 0))],
            [u'oai:ham'])
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(
            from_date=datetime.datetime(2008, 6, 1, 0, 0),
            until_date=datetime.datetime(2010, 6, 1, 0, 0))],
            [u'oai:spam', u'oai:ham'])
        # no matches
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(
            from_date=datetime.datetime(2011, 1, 1, 0, 0))],
            [])
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(
            until_date=datetime.datetime(2008, 1, 1, 0, 0))],
            [])
        # test inclusiveness
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(
            from_date=datetime.datetime(2009, 1, 1, 0, 0),
            )],
            [u'oai:spam', u'oai:ham'])

    def test_oai_query_identifier(self):
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2010, 1, 1, 0, 0, 0),
                              False, {u'spamset':{u'name':u'spam'}},
                              {})
        self.db.flush()
        self.assertEquals(
            [r['id'] for r in self.db.oai_query(identifier=u'oai:spam')],
            [u'oai:spam'])
        
    def test_oai_query_future_dates(self):
        # records with a timestamp in the future should never
        # be returned, this feature can be used to create embargo dates
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2020, 1, 1, 0, 0, 0),
                              False, {u'spamset':{u'name':u'spam'}},
                              {})
        self.db.flush()
        self.assertEquals(list(self.db.oai_query()), [])
        self.assertEquals(list(self.db.oai_query(
            until_date=datetime.datetime(2030, 1, 1, 0, 0, 0))), [])
        self.assertEquals(list(self.db.oai_query(identifier=u'oai:spam')), [])
        
    def test_oai_sets(self):
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2009, 10, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset'),
                                      u'test': dict(name=u'testset')}, {})
        self.db.update_record(u'oai:spamspamspam',
                              datetime.datetime(2009, 6, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset')}, {})
        self.db.update_record(u'oai:ham',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False, {u'ham': dict(name=u'hamset'),
                                      u'test': dict(name=u'testset')}, {})
        self.db.flush()
        # all records
        self.assertEquals([r['id'] for r in self.db.oai_query()],
                          [u'oai:ham', u'oai:spam', u'oai:spamspamspam'])
        # only set ham
        self.assertEquals([r['id'] for r in self.db.oai_query(
            needed_sets=[u'ham'])], [u'oai:ham'])
        # only set spam
        self.assertEquals([r['id'] for r in self.db.oai_query(
            needed_sets=[u'spam'])], [u'oai:spam', u'oai:spamspamspam'])
        # records in spam set and test set
        self.assertEquals([r['id'] for r in self.db.oai_query(
            needed_sets=[u'test', u'spam'])], [u'oai:spam'])
        # only allow records from certain sets
        self.assertEquals([r['id'] for r in self.db.oai_query(
            allowed_sets=[u'test'])], [u'oai:ham', u'oai:spam'])
        self.assertEquals([r['id'] for r in self.db.oai_query(
            allowed_sets=[u'spam', u'ham'])],
                          [u'oai:ham', u'oai:spam', u'oai:spamspamspam'])
        # only allow records from certain sets, combined with set
        self.assertEquals([r['id'] for r in self.db.oai_query(
            allowed_sets=[u'test'], needed_sets=['spam'])],
                          [u'oai:spam'])
        self.assertEquals([r['id'] for r in self.db.oai_query(
            allowed_sets=[u'spam'], needed_sets=['test'])],
                          [u'oai:spam'])
        # certain records should always be disallowed
        self.assertEquals([r['id'] for r in self.db.oai_query(
            disallowed_sets=[u'spam'])],
                           [u'oai:ham'])
        # disallowed sets has precedence over allowed sets
        self.assertEquals([r['id'] for r in self.db.oai_query(
            disallowed_sets=[u'test'], allowed_sets=[u'spam'])],
                           [u'oai:spamspamspam'])
    def test_oai_batching(self):
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2009, 10, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset'),
                                      u'test': dict(name=u'testset')}, {})
        self.db.update_record(u'oai:spamspamspam',
                              datetime.datetime(2009, 6, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset')}, {})
        self.db.update_record(u'oai:ham',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False, {u'ham': dict(name=u'hamset'),
                                      u'test': dict(name=u'testset')}, {})
        self.db.flush()
        self.assertEquals(len(list(self.db.oai_query())), 3)
        self.assertEquals(len(list(self.db.oai_query(batch_size=1))), 1)
        self.assertEquals([r['id'] for r in self.db.oai_query(
            batch_size=1)], [u'oai:ham'])
        self.assertEquals([r['id'] for r in self.db.oai_query(
            batch_size=1, offset=1)], [u'oai:spam'])
        self.assertEquals([r['id'] for r in self.db.oai_query(
            batch_size=1, offset=2)], [u'oai:spamspamspam'])

class ProviderTest(TestCase):
    def setUp(self):
        path = os.path.abspath(os.path.dirname(__file__))
        self.provider = FileBasedContentProvider(
            'file://%s/example*.xml' % path)
        self.db = Database()

    def tearDown(self):
        del self.provider
        del self.db

    def test_provider_update(self):
        self.assertEquals(sorted([id for id in self.provider.update()]),
                          ['example-1234.xml', 'example-2345.xml'])

    def test_provider_content(self):
        self.assertEquals(sorted([id for id in self.provider.update()]),
                          ['example-1234.xml', 'example-2345.xml'])
        for content_id in self.provider.get_content_ids():
            raw_data = self.provider.get_content_by_id(content_id)
            content = ExampleContent(self.provider)
            content.update(raw_data)
            self.db.update_record(content.id,
                                  content.modified,
                                  content.deleted,
                                  content.sets,
                                  content.metadata)
        self.db.flush()
        self.assertEquals(self.db.record_count(), 2)


class ServerTest(TestCase):
    def setUp(self):
        self.db = Database()
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2009, 10, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset'),
                                      u'test': dict(name=u'testset')},
                              {'title': [u'Spam!']})
        self.db.update_record(u'oai:spamspamspam',
                              datetime.datetime(2009, 6, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset')},
                              {'title': [u'Spam Spam Spam!']})
        self.db.update_record(u'oai:ham',
                              datetime.datetime(2010, 10, 13, 12, 30, 0),
                              False, {u'ham': dict(name=u'hamset'),
                                      u'test': dict(name=u'testset')},
                              {'title': [u'Ham!']})
        self.db.flush()
        self.config = FeedConfig('Test Server',
                                 'http://test',
                                 admin_emails=['testuser@localhost'],
                                 metadata_prefixes=['oai_dc', 'mods', 'didl'])
        self.server = Server('http://test', self.db, self.config)
        self.app = MOAIWSGIApp(self.server)
        wsgi_intercept.add_wsgi_intercept('test', 80, lambda : self.app)
        
    def tearDown(self):
        
        wsgi_intercept.remove_wsgi_intercept('test', 80)
        del self.app
        del self.server
        del self.db
        del self.config
        
    def test_identify(self):
        xml = urlopen('http://test?verb=Identify').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai" :"http://www.openarchives.org/OAI/2.0/"})
        self.assertEquals(xpath.string('//oai:repositoryName'),u'Test Server')
        
    def test_list_identifiers(self):
        xml = urlopen('http://test?verb=ListIdentifiers'
                              '&metadataPrefix=oai_dc').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/"})
        self.assertEquals(xpath.strings('//oai:identifier'),
                          [u'oai:ham', u'oai:spam', u'oai:spamspamspam'])

    def test_list_with_dates(self):
        xml = urlopen('http://test?verb=ListIdentifiers'
                              '&metadataPrefix=oai_dc&from=2010-01-01').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/"})
        self.assertEquals(xpath.strings('//oai:identifier'),
                          [u'oai:ham'])
        xml = urlopen('http://test?verb=ListIdentifiers'
                              '&metadataPrefix=oai_dc&until=2010-01-01').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/"})
        self.assertEquals(xpath.strings('//oai:identifier'),
                          [u'oai:spam', u'oai:spamspamspam'])

    def test_list_records(self):
        xml = urlopen('http://test?verb=ListRecords'
                              '&metadataPrefix=oai_dc').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/",
                       "dc": "http://purl.org/dc/elements/1.1/"})
        self.assertEquals(xpath.strings('//oai:identifier'),
                          [u'oai:ham', u'oai:spam', u'oai:spamspamspam'])
        self.assertEquals(xpath.strings('//dc:title'),
                          [u'Ham!', u'Spam!', u'Spam Spam Spam!'])
        xml = urlopen('http://test?verb=ListRecords'
                              '&metadataPrefix=didl').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/",
                       "mods": "http://www.loc.gov/mods/v3"})
        self.assertEquals(xpath.strings('//mods:titleInfo/mods:title'),
                          [u'Ham!', u'Spam!', u'Spam Spam Spam!'])
        
    def test_list_sets(self):
        xml = urlopen('http://test?verb=ListSets').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/"})
        self.assertEquals(sorted(xpath.strings('//oai:setName')),
                          [u'hamset', u'spamset', u'testset'])

    def test_list_hidden_sets(self):
        self.db.update_record(u'oai:spam',
                              datetime.datetime(2009, 10, 13, 12, 30, 0),
                              False, {u'spam': dict(name=u'spamset'),
                                      u'test': dict(name=u'testset',
                                                    hidden=True)},
                              {'title': [u'Spam!']})
        # note that we change the set through the record. It is important
        # that all the records have the same values for each set
        self.db.flush()
        xml = urlopen('http://test?verb=ListSets').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/"})
        # a hidden set should not show up in a listSets request
        self.assertEquals(sorted(xpath.strings('//oai:setName')),
                          [u'hamset', u'spamset'])
        
        # however, we can use the hidden set to filter on
        self.config.sets_disallowed.add(u'test')
        xml = urlopen('http://test?verb=ListIdentifiers'
                              '&metadataPrefix=oai_dc').read()
        doc = etree.fromstring(xml)
        xpath = XPath(doc, nsmap=
                      {"oai": "http://www.openarchives.org/OAI/2.0/"})
        self.assertEquals(xpath.strings('//oai:identifier'),
                          [u'oai:spamspamspam'])


def suite():    
    test_suite = TestSuite()
    test_suite.addTest(makeSuite(XPathUtilTest))
    test_suite.addTest(makeSuite(DatabaseTest))
    test_suite.addTest(makeSuite(ProviderTest))
    test_suite.addTest(makeSuite(ServerTest))
    # note that tests of the oai protocol itself are done in the
    # pyoai codebase
    return test_suite



import re
import uuid

from lxml.builder import ElementMaker

XSI_NS = 'http://www.w3.org/2001/XMLSchema-instance'

class MODS(object):
    """This is the minimods formats as defined by DARE.

    It is registered as prefix 'mods'.'
    """
    
    def __init__(self, prefix, config, db):
        self.prefix = prefix
        self.config = config
        self.db = db

        self.ns = {'mods': 'http://www.loc.gov/mods/v3',
                   'xml':'http://www.w3.org/XML/1998/namespace',
                   'dai': 'info:eu-repo/dai',
                   'gal': 'info:eu-repo/grantAgreement'}

        self.schemas = {
           'mods': 'http://www.loc.gov/standards/mods/v3/mods-3-3.xsd',
           'dai': 'http://purl.org/REP/standards/dai-extension.xsd',
           'gal': 'http://purl.org/REP/standards/gal-extension.xsd'}
        
        
    def get_namespace(self):
        return self.ns[self.prefix]

    def get_schema_location(self):
        return self.schemas[self.prefix]
    
    def __call__(self, element, metadata):

        data = metadata.record
        MODS = ElementMaker(namespace=self.ns['mods'], nsmap=self.ns)
        DAI = ElementMaker(namespace=self.ns['dai'], nsmap=self.ns)
        GAL = ElementMaker(namespace=self.ns['gal'], nsmap=self.ns)
        mods = MODS.mods(version="3.3")
        if data['metadata'].get('identifier'):
            mods.append(MODS.identifier(data['metadata']['identifier'][0],
                                        type="uri"))
        for key, value in data['metadata'].get('identifier_data', {}).items():
            mods.append(MODS.identifier(value, type=key))
                
            
        if data['metadata'].get('title'):
            titleInfo = MODS.titleInfo(
                MODS.title(data['metadata']['title'][0])
                )
            titleInfo.attrib['{%s}lang' % self.ns['xml']] = data['metadata'].get(
                'language', ['en'])[0]
            mods.append(titleInfo)

        mods.append(MODS.typeOfResource('text'))        
        if data['metadata'].get('dare_type'):
            mods.append(MODS.genre(data['metadata']['dare_type'][0]))

        if data['metadata'].get('url'):
            location_el = MODS.location(MODS.url(data['metadata']['url'][0],
                                                 usage="primary display",
                                                 access="object in context"))
            for asset in data['metadata'].get('asset', []):
                if asset.get('access') == 'open':
                    location_el.append(MODS.url(asset['absolute_uri'],
                                                access='raw object'))
            mods.append(location_el)

        public_assets = [a for a in data['metadata'].get('asset', [])
                         if a.get('access') == 'open']
        if len(public_assets) == 1:
            phys_descr_el = MODS.physicalDescription()
            if asset.get('mimetype'):
                phys_descr_el.append(MODS.internetMediaType(asset['mimetype']))
            if asset.get('bytes'):
                try:
                    kbytes = re.sub(r'(\d{3})(?=\d)', r'\1,',
                                    str(int(asset['bytes'])/1024)[::-1])[::-1]
                
                    phys_descr_el.append(MODS.extent('Filesize: %s KB' % kbytes))
                except:
                    pass
                
            phys_descr_el.append(MODS.digitalOrigin('born digital'))
            mods.append(phys_descr_el)
            
            
        if data['metadata'].get('description'):
            mods.append(MODS.abstract(data['metadata']['description'][0]))


        for ctype in ['author', 'editor', 'advisor']:
            contributor_data = []
            for id in data['metadata'].get('%s_rel' % ctype, []):
                contributor = self.db.get_metadata(id)
                contributor['id'] = id
                contributor_data.append(contributor)

            if data['metadata'].get('%s_data' % ctype):
                contributor_data = [s for s in data['metadata'][
                    '%s_data' % ctype]]
        
            if not contributor_data:            
                contributor_data = [{'name':[a]} for a in data[
                    'metadata'].get(ctype, [])]

            dai_list = []
            for contributor in contributor_data:
                contributor_name = contributor.get('name', [''])[0]
                unique_id = uuid.uuid4().hex
                if unique_id[0].isdigit():
                    unique_id = '_'+unique_id
                name = MODS.name(
                    MODS.displayForm(contributor_name),
                    type='personal',
                    ID=unique_id
                    )
                surname = contributor.get('surname')
                if surname:
                    surname = surname[0]
                    prefix = contributor.get('prefix')
                    if prefix:
                        surname = u'%s, %s' % (surname, prefix[0])
                    name.append(MODS.namePart(surname, type="family"))
                initials = contributor.get('initials')
                firstname = contributor.get('firstname')
                if firstname:
                    name.append(MODS.namePart(firstname[0], type="given"))
                elif initials:
                    name.append(MODS.namePart(initials[0], type="given"))
                    
                role = contributor.get('role')
                if role:
                    role = role[0]
                else:
                    roles = {'author': 'aut', 'editor': 'edt', 'advisor':'ths'}
                    role = roles[ctype]
                name.append(                    
                    MODS.role(
                    MODS.roleTerm(role,
                                  type='code',
                                  authority='marcrelator')
                    ))
                mods.append(name)
                dai = contributor.get('dai')
                if dai:
                    dai_list.append((unique_id, dai))
            if dai_list:
                daiList = DAI.daiList()
                daiList.attrib['{%s}schemaLocation' % XSI_NS] = '%s %s' % (
                    self.ns['dai'],
                    self.schemas['dai'])

                for id, dai in dai_list:
                    daiList.append(DAI.identifier(
                        dai[0].split('/')[-1],
                        IDref=id,
                        authority='info:eu-repo/dai/nl'))
                
                mods.append(MODS.extension(daiList))

        for corp in data['metadata'].get('corporate_data', []):
            roles = MODS.role()
            if corp.get('role'):
                roles.append(MODS.roleTerm(corp['role'],
                                           authority="marcrelator",
                                           type="text"))
            if corp.get('role_code'):
                roles.append(MODS.roleTerm(corp['role_code'],
                                           authority="marcrelator",
                                           type="code"))
            mods.append(MODS.name(
                MODS.namePart(corp['name']),
                roles,
                type="corporate"))
            
        if data['metadata'].get('language'):
            lang_el = MODS.language(
                MODS.languageTerm(data['metadata']['language'][0],
                                  type="code",
                                  authority="rfc3066"))
            if data['metadata']['language'][0] == 'en':
                lang_el.append(MODS.languageTerm('English', type="text"))
            if data['metadata']['language'][0] == 'nl':
                lang_el.append(MODS.languageTerm('Nederlands', type="text"))
            mods.append(lang_el)
            
        for host in ['journal', 'series']:
            title = data['metadata'].get('%s_title' % host)
            part_type = {'journal': 'host'}.get(host, host)
            relitem = MODS.relatedItem(type=part_type)
            if title:
                relitem.append(MODS.titleInfo(MODS.title(title[0])))
            else:
                continue
            issn = data['metadata'].get('%s_issn' % host)
            if issn:
                relitem.append(
                    MODS.identifier(issn[0],
                                    type="issn"))
            host_uri = data['metadata'].get('%s_uri' % host)
            if host_uri:
                relitem.append(
                    MODS.identifier(host_uri[0],
                                    type="uri"))
            volume = data['metadata'].get('%s_volume' % host)
            issue = data['metadata'].get('%s_issue' % host)
            start_page = data['metadata'].get('%s_start_page' % host)
            end_page = data['metadata'].get('%s_end_page' % host)
            if volume or issue or end_page or start_page:
                part = MODS.part()
                if volume:
                    part.append(MODS.detail(MODS.number(volume[0]),
                                            type="volume"))
                if issue:
                    part.append(MODS.detail(MODS.number(issue[0]),
                                            type="issue"))
                if start_page or end_page:
                    extent = MODS.extent(unit="page")
                    if start_page:
                        extent.append(MODS.start(start_page[0]))
                    if end_page:
                        extent.append(MODS.end(end_page[0]))
                    part.append(extent)
                relitem.append(part)
            if data['metadata'].get('%s_publisher' % host):
                relitem.append(
                    MODS.originInfo(
                      MODS.publisher(
                        data['metadata']['%s_publisher' % host][0])))
                
            mods.append(relitem)

        origin = MODS.originInfo()
        mods.append(origin)
        if data['metadata'].get('publisher'):
            origin.append(MODS.publisher(data['metadata']['publisher'][0]))
        if data['metadata'].get('date'):
            origin.append(MODS.dateIssued(data['metadata']['date'][0],
                                        encoding='w3cdtf'))

        
        classifications = data['metadata'].get('classification', [])
        for classification in classifications:
            if classification.count('#') == 1:
                authority, value = classification.split('#')
                mods.append(MODS.classification(value, authority=authority))
            else:
                mods.append(MODS.classification(classification))
        
        subjects = data['metadata'].get('subject', [])
        if subjects:
            s_el = MODS.subject()
            for subject in subjects:
                s_el.append(MODS.topic(subject))
            mods.append(s_el)

        if data['metadata'].get('rights'):
            mods.append(MODS.accessCondition(data['metadata']['rights'][0]))
        
        projects = data['metadata'].get('project', [])
        funders = set([prj['funder'] for prj in projects if prj.get('funder')])
        funder_ids = {}
        for funder in funders:
            unique_id = uuid.uuid4().hex
            if unique_id[0].isdigit():
                unique_id = '_'+unique_id
            funder_ids[funder] = unique_id
            mods.append(
                MODS.name(MODS.namePart(funder),
                          MODS.role(MODS.roleTerm('fnd',
                                                  authority='marcrelator',
                                                  type='code')),
                          ID=unique_id,
                          type='corporate'))
        
        if projects:
            galList = GAL.grantAgreementList()
            galList.attrib['{%s}schemaLocation' % XSI_NS] = '%s %s' % (
                self.ns['gal'],
                self.schemas['gal'])
            mods.append(MODS.extension(galList))
            for prj in projects:
                el = GAL.grantAgreement(code=prj['id'])
                if prj.get('funder'):
                    el.append(GAL.funder(IDref=funder_ids[prj['funder']]))
                if prj.get('title'):
                    el.append(GAL.title(prj['title']))
                galList.append(el)

        info = data['metadata'].get('record_info_data', {})
        if info:
            record_info_el = MODS.recordInfo()
            if info.get('source'):
                record_info_el.append(MODS.recordContentSource(info['source']))
            if info.get('identifier'):
                record_info_el.append(MODS.recordIdentifier(info['identifier']))
            for key, value in info.get('identifier_data', {}).items():
                record_info_el.append(MODS.recordIdentifier(value, source=key))
            if info.get('origin'):
                record_info_el.append(MODS.recordOrigin(info['origin']))
            if info.get('created'):
                record_info_el.append(
                    MODS.recordCreationDate(info['created'],
                                            encoding="w3cdtf"))
            if info.get('changed'):
                record_info_el.append(
                    MODS.recordChangeDate(info['changed'],
                                          encoding="w3cdtf"))
            mods.append(record_info_el)
            
        mods.attrib['{%s}schemaLocation' % XSI_NS] = '%s %s' % (
            self.ns['mods'],
            self.schemas['mods'])
        
        element.append(mods)

class NL_MODS(MODS):
    """
    like mods, but dateIssued uses wrong iso8601 encoding instead of w3cdtf
    """
    def __init__(self, prefix, config, db):
        super(NL_MODS, self).__init__(prefix, config, db)
        self.ns['nl_mods'] = self.ns['mods']
        self.schemas['nl_mods'] = self.schemas['mods']
        
    def __call__(self, element, metadata):
        super(NL_MODS, self).__call__(element, metadata)
        for el in element.xpath(
            './mods:mods/mods:originInfo/'
            'mods:dateIssued[@encoding="w3cdtf"]', namespaces=self.ns):
            el.attrib['encoding'] = 'iso8601'
        
        



"""
moai.server
===========

The Server module contains implementations
of :ref:`IServer` and :ref:`IFeedConfig`.

"""
import os
import tempfile

import oaipmh.error

from moai.oai import OAIServerFactory, OAIServer

class Server(object):
    """This is the default implementation of the
    :ref:`IServer` interface. 

    Developers might want to subclass this, to provide custom
    asset handling in their implementation.
    """
    def __init__(self, base_url, db, config):
        self.base_url = base_url
        self._db = db
        self._config = config

    def download_asset(self, req, url, config):
        """Download an asset
        """
        url = url.lstrip('/')
        asset_url = url.split('asset/')[-1]
        id, filename = asset_url.split('/')

        for asset in self._db.get_assets(id):
            if (asset['filename'] == filename or
                asset['md5'] == filename):
                break
        else:
            return req.send_status(
                '404 File not Found',
                'The asset "%s" does not exist' % filename)
            
        if not os.path.isfile(asset['path']):
            return req.send_status(
                '404 File not Found',
                'The asset file "%s" does not exist' % filename)

        return req.send_file(asset['path'],
                             asset['mimetype'].encode('ascii'))

    def allow_download(self, url, config):
        """Returns a boolean indicating if it is okay to download an
        asset or not. 

        By examining the url, the id of the oai record can be found, and
        thus the metadata can be accessed. This metadata could have settings
        to indicate that the asset is private and should not be downloaded

        """
        
        url = url.lstrip('/')
        asset_url = url.split('asset/')[-1]
        id, filename = asset_url.split('/')
        
        oai_server = OAIServer(self._db, config)
        try:
            header, metadata, description = oai_server.getRecord(
                'oai_dc', config.oai_id_prefix + id)
        except oaipmh.error.IdDoesNotExistError:
            # record is not in the oai feed, don't download
            return False
        if header.isDeleted():
            # record has deleted status, don't download
            return False

        return True

    def is_asset_url(self, url, config):
        """Returns a boolean indicating if this is a url
        for downloading an asset or not
        """
        
        if url.startswith('asset/'):
            return True
        return False
            
    def handle_request(self, req):
        if not req.url().startswith(self.base_url):
            return req.send_status(
                '500 Internal Server Error',
                'The url "%s" does not start with base url "%s".' % (
                req.url(), self.base_url))
        url = req.url()[len(self.base_url):]
        
        if url.startswith('/'):
            url = url[1:]
        if url.endswith('/'):
            url = url[:-1]
                                   
        urlparts = url.split('/')                           

        if len(urlparts) == 0:
            return req.send_status('500 Internal Server Error',
                 'No server was selected, please append server name to url.')
        
            
        url = '/'.join(urlparts)

        if self.is_asset_url(url, self._config):
            if self.allow_download(url, self._config):
                return self.download_asset(req, url, self._config)
            else:
                return req.send_status('403 Forbidden',
                                       'You are not allowed to download this asset')

        oai_server = OAIServerFactory(self._db, self._config)
        return req.write(oai_server.handleRequest(req.query_dict()), 'text/xml')

class FeedConfig(object):
    """The feedconfig object contains all the settings for a specific
    feed. It implements the :ref:`IFeedConfig` interface.
    """
    def __init__(self,
                 repository_name,
                 base_url,
                 admin_emails = None,
                 metadata_prefixes = None,
                 batch_size = 100,
                 content_type = None,
                 sets_needed = None,
                 sets_allowed = None,
                 sets_disallowed = None,
                 sets_deleted = None,
                 filter_sets = None,
                 extra_args = None):
        extra_args = extra_args or {}
        self.name = repository_name
        self.url = base_url
        self.admins = admin_emails or []
        self.metadata_prefixes = metadata_prefixes or ['oai_dc']
        self.batch_size = batch_size
        self.content_type = content_type
        self.sets_needed = set(sets_needed or [])
        self.sets_allowed = set(sets_allowed or [])
        self.sets_disallowed = set(sets_disallowed or [])
        self.sets_deleted = set(sets_deleted or [])
        self.filter_sets = set(filter_sets or [])
        self.delay = extra_args.get('delay', 0)
        self.base_asset_path = extra_args.get('base_asset_path',
                                              tempfile.gettempdir())
        self.oai_id_prefix = extra_args.get('oai_id_prefix', '')
        




from lxml.builder import ElementMaker

XSI_NS = 'http://www.w3.org/2001/XMLSchema-instance'
  
class OAIDC(object):
    """The standard OAI Dublin Core metadata format.
    
    Every OAI feed should at least provide this format.

    It is registered under the name 'oai_dc'
    """
    
    def __init__(self, prefix, config, db):
        self.prefix = prefix
        self.config = config
        self.db = db

        self.ns = {'oai_dc': 'http://www.openarchives.org/OAI/2.0/oai_dc/',
                   'dc':'http://purl.org/dc/elements/1.1/'}
        self.schemas = {
            'oai_dc': 'http://www.openarchives.org/OAI/2.0/oai_dc.xsd'}
        
    def get_namespace(self):
        return self.ns[self.prefix]

    def get_schema_location(self):
        return self.schemas[self.prefix]
    
    def __call__(self, element, metadata):

        data = metadata.record
        
        OAI_DC =  ElementMaker(namespace=self.ns['oai_dc'],
                               nsmap =self.ns)
        DC = ElementMaker(namespace=self.ns['dc'])

        oai_dc = OAI_DC.dc()
        oai_dc.attrib['{%s}schemaLocation' % XSI_NS] = '%s %s' % (
            self.ns['oai_dc'],
            self.schemas['oai_dc'])

        for field in ['title', 'creator', 'subject', 'description',
                      'publisher', 'contributor', 'type', 'format',
                      'identifier', 'source', 'language', 'date',
                      'relation', 'coverage', 'rights']:
            el = getattr(DC, field)
            for value in data['metadata'].get(field, []):
                if field == 'identifier' and data['metadata'].get('url'):
                    value = data['metadata']['url'][0]
                oai_dc.append(el(value))
        
        element.append(oai_dc)



import sys

class ContentError(Exception):
    def __init__(self, contentclass, input):
        err, detail, tb = sys.exc_info()
        self.err= err
        self.detail = detail
        self.tb = tb
        self.contentclass = contentclass
        self.input = input
        
    def logmessage(self):
        return 'Can not create %s: %s \n         %s\n         %s' % (
            self.contentclass.__name__,
            self.err.__name__,
            self.detail,
            self.input)

class DatabaseError(Exception):
    def __init__(self, id, add_type):
        err, detail, tb = sys.exc_info()
        self.err= err
        self.detail = detail
        self.tb = tb
        self.id = id
        self.add_type = add_type
    def logmessage(self):
        return 'Can not add %s "%s" to database: %s\n         %s' % (
            self.add_type,
            self.id,
            self.err.__name__,
            self.detail)
    
class UnknownRecordID(Exception):
    pass




from lxml.builder import ElementMaker

from moai.metadata.didl import DIDL, XSI_NS
        
class DareDIDL(DIDL):
    """A metadata prefix implementing the DARE DIDL metadata format
    this format is registered under the name "didl"
    Note that this format re-uses oai_dc and mods formats that come with
    MOAI by default
    """    

    def __init__(self, prefix, config, db):
        super(DareDIDL, self).__init__(prefix, config, db)
        self.prefix = 'didl'

    def __call__(self, element, metadata):
        super(DareDIDL, self).__call__(element, metadata)
        data = metadata.record

        DIDL = ElementMaker(namespace=self.ns['didl'], nsmap=self.ns)
        DII = ElementMaker(namespace=self.ns['dii'])
        DIP = ElementMaker(namespace=self.ns['dip'])
        
        didl_item = element.getchildren()[0].getchildren()[0]

        didl_item.insert(0, 
           DIDL.Descriptor(
            DIDL.Statement(
            DII.Identifier(data['metadata'].get('dare_id', [''])[0]),
            mimeType="application/xml")
            ))



from pkg_resources import iter_entry_points

from datetime import datetime
import pkg_resources
import time

import oaipmh
import oaipmh.metadata
import oaipmh.server
import oaipmh.error

def get_writer(prefix, config, db):
    for writer in iter_entry_points(group='moai.format', name=prefix):
        return writer.load()(prefix, config, db)
    else:
        raise ValueError('No such metadata format registered: %s' % prefix)


class OAIServer(object):
    """An OAI-2.0 compliant oai server.
    
    Underlying code is based on pyoai's oaipmh.server'
    """
    
    def __init__(self, db, config):
        self.db = db
        self.config = config

    def identify(self):
        result = oaipmh.common.Identify(
            repositoryName=self.config.name,
            baseURL=self.config.url,
            protocolVersion='2.0',
            adminEmails=self.config.admins,
            earliestDatestamp=self.db.oai_earliest_datestamp(),
            deletedRecord='transient',
            granularity='YYYY-MM-DDThh:mm:ssZ',
            compression=['identity'],
            toolkit_description=False)

        version = ''
        pyoai_egg = pkg_resources.working_set.find(
            pkg_resources.Requirement.parse('pyoai'))
        moai_egg = pkg_resources.working_set.find(
            pkg_resources.Requirement.parse('MOAI'))
        
        if moai_egg and pyoai_egg:
            version = '<version>%s (using pyoai%s)</version>' % (
                moai_egg.version,
                pyoai_egg.version)
        result.add_description(
            '<toolkit xsi:schemaLocation='
            '"http://oai.dlib.vt.edu/OAI/metadata/toolkit '
            'http://oai.dlib.vt.edu/OAI/metadata/toolkit.xsd" '
            'xmlns="http://oai.dlib.vt.edu/OAI/metadata/toolkit" '
            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">'
            '<title>MOAI</title>'
            '%s'
            '<URL>http://moai.infrae.com</URL>'
            '</toolkit>' % version)
            
        return result

    def listMetadataFormats(self, identifier=None):
        result = []
        for prefix in self.config.metadata_prefixes:
            writer = get_writer(prefix, self.config, self.db)
            ns = writer.get_namespace()
            schema = writer.get_schema_location()
            result.append((prefix, schema, ns))
        return result
    
    def listSets(self, cursor=0, batch_size=20):
        for set in self.db.oai_sets(cursor, batch_size):
            yield [set['id'], set['name'], set['description']]

    def listRecords(self, metadataPrefix, set=None, from_=None, until=None,
                    cursor=0, batch_size=10):
        
        self._checkMetadataPrefix(metadataPrefix)
        for record in self._listQuery(set, from_, until, cursor, batch_size):
            header, metadata = self._createHeaderAndMetadata(record)
            yield header, metadata, None

    def listIdentifiers(self, metadataPrefix, set=None, from_=None, until=None,
                        cursor=0, batch_size=10):
        
        self._checkMetadataPrefix(metadataPrefix)
        for record in self._listQuery(set, from_, until, cursor, batch_size):
            yield self._createHeader(record)

    def getRecord(self, metadataPrefix, identifier):
        self._checkMetadataPrefix(metadataPrefix)
        header = None
        metadata = None
        for record in self._listQuery(identifier=identifier):
            header, metadata = self._createHeaderAndMetadata(record)
        if header is None:
            raise oaipmh.error.IdDoesNotExistError(identifier)
        return header, metadata, None
        
    def _checkMetadataPrefix(self, metadataPrefix):
        if metadataPrefix not in self.config.metadata_prefixes:
            raise oaipmh.error.CannotDisseminateFormatError

    def _createHeader(self, record):
        deleted = record['deleted']
        for setspec in record['sets']:
            if setspec in self.config.sets_deleted:
                deleted = True
                break
        return oaipmh.common.Header(None,
                                    record['id'],
                                    record['modified'],
                                    record['sets'],
                                    deleted)

    def _createHeaderAndMetadata(self, record):
        header = self._createHeader(record)
        metadata = oaipmh.common.Metadata(None, record)
        metadata.record = record
        return header, metadata
    
    def _listQuery(self, set=None, from_=None, until=None, 
                   cursor=0, batch_size=10, identifier=None):
            
        now = datetime.utcnow()
        if until != None and until > now:
            # until should never be in the future
            until = now
            
        if self.config.delay:
            # subtract delay from until_ param, if present
            if until is None:
                until = datetime.utcnow()
            until = until.timetuple()
            ut = time.mktime(until)-self.filter_data.delay
            until = datetime.fromtimestamp(ut)
            
        needed_sets = self.config.sets_needed.copy()
        if not set is None:
            needed_sets.add(set)
        allowed_sets = self.config.sets_allowed.copy()
        disallowed_sets = self.config.sets_disallowed.copy()    
        
        return self.db.oai_query(offset=cursor,
                                 batch_size=batch_size,
                                 needed_sets=needed_sets,
                                 disallowed_sets=disallowed_sets,
                                 allowed_sets=allowed_sets,
                                 from_date=from_,
                                 until_date=until,
                                 identifier=identifier
                                 )

def OAIServerFactory(db, config):
    """Create a new OAI batching OAI Server given a config and
    a database"""
    
    metadata_registry = oaipmh.metadata.MetadataRegistry()
    for prefix in config.metadata_prefixes:
        metadata_registry.registerWriter(prefix,
                                         get_writer(prefix, config, db))
            
    return oaipmh.server.BatchingServer(
        OAIServer(db, config),
        metadata_registry=metadata_registry,
        resumption_batch_size=config.batch_size
        )



import os

from zope.interface import implements
from oaipmh.client import Client
from oaipmh.metadata import MetadataRegistry
from oaipmh.error import NoRecordsMatchError
from lxml import etree

from moai.interfaces import IContentProvider
from moai.provider.file import FileBasedContentProvider

class OAIBasedContentProvider(FileBasedContentProvider):
    """Providers content by harvesting OAI feeds.
    Implements the :ref:`IContentProvider` interface
    """
    
    implements(IContentProvider)

    def __init__(self, oai_url, output_path, metadata_prefix='oai_dc'):
        super(OAIBasedContentProvider, self).__init__(output_path, '*.xml')
        self._url = oai_url
        self._prefix = metadata_prefix

    def set_logger(self, log):
        self._log = log

    def update(self, from_date=None):
        self._log.info('Harvesting oai server: %s' % self._url)
        registry = MetadataRegistry()
        registry.registerReader(self._prefix, lambda el: el)

        client = Client(self._url, registry)
        try:
            for header, element, about in client.listRecords(
                metadataPrefix = self._prefix,
                from_ = from_date):
                added = self._process_record(header, element)
                if added:
                    yield self._get_id(header)
        except NoRecordsMatchError:
            pass

        super(OAIBasedContentProvider, self).update()

    def _get_id(self, header):
        return header.identifier()

    def _process_record(self, header, element):

        oai_id = self._get_id(header)
        path = os.path.join(self.path, '%s.xml' % oai_id)
        fp = open(path, 'w')
        fp.write(etree.tostring(element, encoding="utf8"))
        fp.close()
        return True




import os

from webob import Request, Response

from moai.server import Server, FeedConfig
from moai.database import get_database

class WSGIRequest(object):
    """This is a request object that can be used in a WSGI environment.
    It implements :ref:`IServerRequest` interface.
    """
    def __init__(self, request):
        self._req = request

    def url(self):
        return self._req.url
    
    def redirect(self, url):
        """Redirect to this url
        """
        response = Response()
        response.status = 302
        response.location = url
        return response

    def send_file(self, path, mimetype):
        """Send the file located at 'path' back to the user
        """
        response = Response(content_type=mimetype,
                            conditional_response=True)
        response.last_modified = os.path.getmtime(path)
        response.app_iter = FileIterable(path)
        with open(path) as f:
            response.body = f.read()
        response.content_length = os.path.getsize(path)
        # do not accept ranges, since this does not work reliable
        # with acrobat IE plugin
        response.headers['Accept-Ranges'] = 'none'
        return response
    
    def query_dict(self):
        """Return a dictionary with QueryString values of the
        request
        """
        args = dict(self._req.GET)
        args.update(dict(self._req.POST))
        return args

    def write(self, data, mimetype):
        """Write data back to the client
        """
        response = Response()
        response.content_type = mimetype
        response.body = data
        return response

    def send_status(self, code, msg='', mimetype='text/plain'):
        response = Response()
        response.content_type = mimetype
        response.status = int(code.split()[0])
        response.body = msg
        return response


class MOAIWSGIApp(object):
    # the wsgi app, calls the IServer with the IServerRequest 
    def __init__(self, server):
        self.server = server
        
    def __call__(self, environ, start_response):
        request = Request(environ)
        response = self.server.handle_request(WSGIRequest(request))
        return response(environ, start_response)

def app_factory(global_config,
                name,
                url,
                admin_email,
                database,
                formats,
                **kwargs):
    # WSGI APP Factory
    formats = formats.split()
    admin_email = admin_email.split()
    sets_deleted = kwargs.get('deleted_sets') or []
    if sets_deleted:
        sets_deleted = sets_deleted.split()
    sets_disallowed = kwargs.get('disallowed_sets', '') or []
    if sets_disallowed:
        sets_disallowed = sets_disallowed.split()
    sets_allowed = kwargs.get('allowed_sets', '') or []
    if sets_allowed:
        sets_allowed = sets_allowed.split()
    sets_needed = kwargs.get('needed_sets', '') or []
    if sets_needed:
        sets_needed = sets_needed.split()
    database = get_database(database, kwargs)
    feedconfig = FeedConfig(name,
                            url,
                            admin_emails=admin_email,
                            metadata_prefixes=formats,
                            sets_deleted=sets_deleted,
                            sets_disallowed=sets_disallowed,
                            sets_allowed=sets_allowed,
                            sets_needed=sets_needed,
                            extra_args=kwargs)
    server = Server(url, database, feedconfig)
    
    return MOAIWSGIApp(server)

class FileIterable(object):
    # Helper objects to stream asset files
    def __init__(self, filename, start=None, stop=None):
        self.filename = filename
        self.start = start
        self.stop = stop
    def __iter__(self):
        return FileIterator(self.filename, self.start, self.stop)
    def app_iter_range(self, start, stop):
        return self.__class__(self.filename, start, stop)

class FileIterator(object):
    chunk_size = 4096
    def __init__(self, filename, start, stop):
        self.filename = filename
        self.fileobj = open(self.filename, 'rb')
        if start:
            self.fileobj.seek(start)
        if stop is not None:
            self.length = stop - start
        else:
            self.length = None
    def __iter__(self):
        return self
    def next(self):
        if self.length is not None and self.length <= 0:
            raise StopIteration
        chunk = self.fileobj.read(self.chunk_size)
        if not chunk:
            raise StopIteration
        if self.length is not None:
            self.length -= len(chunk)
            if self.length < 0:
                # Chop off the extra:
                chunk = chunk[:self.length]
        return chunk




from lxml import etree

from moai.utils import XPath

class ExampleContent(object):
    def __init__(self, provider):
        self.provider = provider
        self.id = None
        self.modified = None
        self.deleted = None
        self.sets = None
        self.metadata = None

    def update(self, path):
        doc = etree.parse(path)
        xpath = XPath(doc, nsmap={'x':'http://example.org/data'})
        
        self.root = doc.getroot()

        id = xpath.string('//x:id')
        self.id = 'oai:example-%s' % id
        self.modified = xpath.date('//x:modified')
        self.deleted = False

        author_data = []
        for num, el in enumerate(xpath('//x:author'), 1):
            first = xpath.string('//x:author[%d]/x:givenName' % num)
            sur = xpath.string('//x:author[%d]/x:familyName' % num)
            name = u'%s %s' % (first, sur)
            author_data.append({'name': [name],
                                'surname': [sur],
                                'firstname': [first],
                                'role': [u'aut']})

        self.metadata = {'identifier': [u'http://example.org/data/%s' % id],
                         'title': [xpath.string('//x:title')],
                         'subject': xpath.strings('//x:subject'),
                         'description': [xpath.string('//x:abstract')],
                         'creator': [d['name'][0] for d in author_data],
                         'author_data': author_data,
                         'language': [u'en'],
                         'date': [xpath.string('//x:issued')]}
        
        self.sets = {u'example': {u'name':u'example',
                                  u'description':u'An Example Set'}}

        access = xpath.string('//x:access')
        if access == 'public':
            self.sets[u'public'] = {u'name':u'public',
                                    u'description':u'Public access'}
            self.metadata['rights'] = [u'open access']
        elif access == 'private':
            self.sets[u'private'] = {u'name':u'private',
                                     u'description':u'Private access'}
            self.metadata['rights'] = [u'restricted access']




import time

class ListBasedContentProvider(object):
    """Provides content from a python list,
    implementation of :ref:`IContentProvider`"""

    def __init__(self, content):
        self._content = content

    def set_logger(self, log):
        self._log = log

    def update(self, from_date=None):
        return self.get_content_ids(from_date)

    def count(self):
        return len(self._content)

    def get_content_ids(self, from_date=None):
        if from_date is None:
            return range(len(self._content))
        else:
            from_time = time.mktime(from_date.timetuple())

            result = []
            for id in range(len(self._content)):
                when_mod = self.get_content_by_id(id).get('when_modified', '')

                if when_mod != '':
                    when_mod = time.mktime(when_mod.timetuple())
                
                    if from_time <= when_mod:
                        result.append(id)

            return result

    def get_content_by_id(self, id):
        return self._content[id]






import datetime
import json
from pkg_resources import iter_entry_points

import six
import sqlalchemy as sql

from moai.utils import check_type

def get_database(uri, config=None):
    prefix = uri.split(':')[0]
    for entry_point in iter_entry_points(group='moai.database', name=prefix):
        dbclass = entry_point.load()
        try:
            return dbclass(uri, config)
        except TypeError:
            # ugly backwards compatibility hack
            return dbclass(uri)
    raise ValueError('No such database registered: %s' % prefix)


class SQLDatabase(object):
    """Sql implementation of a database backend
    This implements the :ref:`IDatabase` interface, look there for
    more documentation.
    """

    def __init__(self, dburi=None):
        self._uri = dburi
        self._db = self._connect()
        self._records = self._db.tables['records']
        self._sets = self._db.tables['sets']
        self._setrefs = self._db.tables['setrefs']
        self._reset_cache()
        
    def _connect(self):
        dburi = self._uri
        if dburi is None:
            dburi = 'sqlite:///:memory:'
            
        engine = sql.create_engine(dburi)
        db = sql.MetaData(engine)
        
        sql.Table('records', db,
                  sql.Column('record_id', sql.Unicode, primary_key=True),
                  sql.Column('modified', sql.DateTime, index=True),
                  sql.Column('deleted', sql.Boolean),
                  sql.Column('metadata', sql.String))
        
        sql.Table('sets', db,
                  sql.Column('set_id', sql.Unicode, primary_key=True),
                  sql.Column('hidden', sql.Boolean),
                  sql.Column('name', sql.Unicode),
                  sql.Column('description', sql.Unicode))

        sql.Table('setrefs', db,
                  sql.Column('record_id', sql.Integer, 
                             sql.ForeignKey('records.record_id'),
                             index=True, primary_key=True),
                  sql.Column('set_id', sql.Integer,
                             sql.ForeignKey('sets.set_id'),
                             index=True, primary_key=True))
        
        db.create_all()
        return db

    def flush(self):
        oai_ids = set()
        for row in sql.select([self._records.c.record_id]).execute():
            oai_ids.add(row[0])
        for row in sql.select([self._sets.c.set_id]).execute():
            oai_ids.add(row[0])

        deleted_records = []
        deleted_sets = []
        deleted_setrefs = []

        inserted_records = []
        inserted_sets = []
        inserted_setrefs = []

        
        for oai_id, item in self._cache['records'].items():
            if oai_id in oai_ids:
                # record allready exists
                deleted_records.append(oai_id)
            item['record_id'] = oai_id
            inserted_records.append(item)

        for oai_id, item in self._cache['sets'].items():
            if oai_id in oai_ids:
                # set allready exists
                deleted_sets.append(oai_id)
            item['set_id'] = oai_id
            inserted_sets.append(item)

        for record_id, set_ids in self._cache['setrefs'].items():
            deleted_setrefs.append(record_id)
            for set_id in set_ids:
                inserted_setrefs.append(
                    {'record_id':record_id, 'set_id': set_id})

        # delete all processed records before inserting
        if deleted_records:
            self._records.delete(
                self._records.c.record_id == sql.bindparam('record_id')
                ).execute(
                [{'record_id': rid} for rid in deleted_records])
        if deleted_sets:
            self._sets.delete(
                self._sets.c.set_id == sql.bindparam('set_id')
                ).execute(
                [{'set_id': sid} for sid in deleted_sets])
        if deleted_setrefs:
            self._setrefs.delete(
                self._setrefs.c.record_id == sql.bindparam('record_id')
                ).execute(
                [{'record_id': rid} for rid in deleted_setrefs])

        # batch inserts
        if inserted_records:
            self._records.insert().execute(inserted_records)
        if inserted_sets:
            self._sets.insert().execute(inserted_sets)
        if inserted_setrefs:
            self._setrefs.insert().execute(inserted_setrefs)

        self._reset_cache()

    def _reset_cache(self):
        self._cache = {'records': {}, 'sets': {}, 'setrefs': {}}
        
            
    def update_record(self, oai_id, modified, deleted, sets, metadata):
        # adds a record, call flush to actually store in db

        check_type(oai_id,
                   six.text_type,
                   prefix="record %s" % oai_id,
                   suffix='for parameter "oai_id"')
        check_type(modified,
                   datetime.datetime,
                   prefix="record %s" % oai_id,
                   suffix='for parameter "modified"')
        check_type(deleted,
                   bool,
                   prefix="record %s" % oai_id,
                   suffix='for parameter "deleted"')
        check_type(sets,
                   dict,
                   unicode_values=True,
                   recursive=True,
                   prefix="record %s" % oai_id,
                   suffix='for parameter "sets"')
        check_type(metadata,
                   dict,
                   prefix="record %s" % oai_id,
                   suffix='for parameter "metadata"')

        def date_handler(obj):
            if hasattr(obj, 'isoformat'):
                return obj.isoformat()
            else:
                raise TypeError('Object of type %s with value of %s is not JSON serializable' % (type(obj), repr(obj)))

        metadata = json.dumps(metadata, default=date_handler)
        self._cache['records'][oai_id] = (dict(modified=modified,
                                               deleted=deleted,
                                               metadata=metadata))
        self._cache['setrefs'][oai_id] = []
        for set_id in sets:
            self._cache['sets'][set_id] = dict(
                name = sets[set_id]['name'],
                description = sets[set_id].get('description'),
                hidden = sets[set_id].get('hidden', False))
            self._cache['setrefs'][oai_id].append(set_id)
            
    def get_record(self, oai_id):
        row = self._records.select(
            self._records.c.record_id == oai_id).execute().fetchone()
        if row is None:
            return
        record = {'id': row.record_id,
                  'deleted': row.deleted,
                  'modified': row.modified,
                  'metadata': json.loads(row.metadata),
                  'sets': self.get_setrefs(oai_id)}
        return record

    def get_set(self, oai_id):
        row = self._sets.select(
            self._sets.c.set_id == oai_id).execute().fetchone()
        if row is None:
            return
        return {'id': row.set_id,
                'name': row.name,
                'description': row.description,
                'hidden': row.hidden}

    def get_setrefs(self, oai_id, include_hidden_sets=False):
        set_ids = []
        query = sql.select([self._setrefs.c.set_id])
        query.append_whereclause(self._setrefs.c.record_id == oai_id)
        if include_hidden_sets == False:
            query.append_whereclause(
                sql.and_(self._sets.c.set_id == self._setrefs.c.set_id,
                         self._sets.c.hidden == include_hidden_sets))
        
        for row in query.execute():
            set_ids.append(row[0])
        set_ids.sort()
        return set_ids

    def record_count(self):
        return sql.select([sql.func.count('*')],
                          from_obj=[self._records]).execute().fetchone()[0]

    def set_count(self):
        return sql.select([sql.func.count('*')],
                          from_obj=[self._sets]).execute().fetchone()[0]
        
    def remove_record(self, oai_id):
        self._records.delete(
            self._records.c.record_id == oai_id).execute()
        self._setrefs.delete(
            self._setrefs.c.record_id == oai_id).execute()

    def remove_set(self, oai_id):
        self._sets.delete(
            self._sets.c.set_id == oai_id).execute()
        self._setrefs.delete(
            self._setrefs.c.set_id == oai_id).execute()

    def oai_sets(self, offset=0, batch_size=20):
        for row in self._sets.select(
              self._sets.c.hidden == False
            ).offset(offset).limit(batch_size).execute():
            yield {'id': row.set_id,
                   'name': row.name,
                   'description': row.description}

    def oai_earliest_datestamp(self):
        row = sql.select([self._records.c.modified],
                         order_by=[sql.asc(self._records.c.modified)]
                         ).limit(1).execute().fetchone()
        if row:
            return row[0]
        return datetime.datetime(1970, 1, 1)
    
    def oai_query(self,
                  offset=0,
                  batch_size=20,
                  needed_sets=None,
                  disallowed_sets=None,
                  allowed_sets=None,
                  from_date=None,
                  until_date=None,
                  identifier=None):

        needed_sets = needed_sets or []
        disallowed_sets = disallowed_sets or []
        allowed_sets = allowed_sets or []
        if batch_size < 0:
            batch_size = 0

        # make sure until date is set, and not in future
        if until_date == None or until_date > datetime.datetime.utcnow():
            until_date = datetime.datetime.utcnow()


        query = self._records.select(
            order_by=[sql.desc(self._records.c.modified)])

        # filter dates
        query.append_whereclause(self._records.c.modified <= until_date)

        if not identifier is None:
            query.append_whereclause(self._records.c.record_id == identifier)

        if not from_date is None:
            query.append_whereclause(self._records.c.modified >= from_date)

        # filter sets

        setclauses = []
        for set_id in needed_sets:
            alias = self._setrefs.alias()
            setclauses.append(
                sql.and_(
                alias.c.set_id == set_id,
                alias.c.record_id == self._records.c.record_id))
            
        if setclauses:
            query.append_whereclause((sql.and_(*setclauses)))
            
        allowed_setclauses = []
        for set_id in allowed_sets:
            alias = self._setrefs.alias()
            allowed_setclauses.append(
                sql.and_(
                alias.c.set_id == set_id,
                alias.c.record_id == self._records.c.record_id))
            
        if allowed_setclauses:
            query.append_whereclause(sql.or_(*allowed_setclauses))

        disallowed_setclauses = []
        for set_id in disallowed_sets:
            alias = self._setrefs.alias()
            disallowed_setclauses.append(
                sql.exists([self._records.c.record_id],
                           sql.and_(
                alias.c.set_id == set_id,
                alias.c.record_id == self._records.c.record_id)))
            
        if disallowed_setclauses:
            query.append_whereclause(sql.not_(sql.or_(*disallowed_setclauses)))
            
        for row in query.distinct().offset(offset).limit(batch_size).execute():
            yield {'id': row.record_id,
                   'deleted': row.deleted,
                   'modified': row.modified,
                   'metadata': json.loads(row.metadata),
                   'sets': self.get_setrefs(row.record_id)
                   }




# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages
from os.path import join, dirname

setup(
    name='MOAI-iplweb',
    version='2.0.0',
    author=u'Michał Pasternak',
    author_email='michal.dtz@gmail.com',
    url='http://iplweb.pl/',
    description="MOAI, A Open Access Server Platform for Institutional Repositories",
    long_description=(open(join(dirname(__file__), 'ABOUT.txt')).read()+
                      '\n'+
                      open(join(dirname(__file__), 'README.rst')).read()+
                      '\n'+
                      open(join(dirname(__file__), 'HISTORY.txt')).read()),
    classifiers=["Development Status :: 5 - Production/Stable",
                 "Programming Language :: Python",
                 "License :: OSI Approved :: BSD License",
                 "Topic :: Software Development :: Libraries :: Python Modules",
                 "Environment :: Web Environment"],
    packages=find_packages(),
    include_package_data = True,
    zip_safe=False,
    license='BSD',
    entry_points= {
    'console_scripts': [
        'update_moai = moai.tools:update_moai',
      ],
    'paste.app_factory':[
        'main=moai.wsgi:app_factory'
     ],
    'moai.content':[
        'moai_example=moai.example:ExampleContent'
     ],
    'moai.database':[
        'sqlite=moai.database:SQLDatabase',
        'mysql=moai.database:SQLDatabase',
        'postgres=moai.database:SQLDatabase',
        'oracle=moai.database:SQLDatabase'],
    'moai.provider':[
        'file=moai.provider.file:FileBasedContentProvider',
        'list=moai.provider.list:ListBasedContentProvider',
        'oai=moai.provider.oai:OAIBasedContentProvider',
        'fedora=moai.provider.feadora:FedoraBasedContentProvider'
     ],
    'moai.format':[
         'oai_dc=moai.metadata.oaidc:OAIDC',
         'mods=moai.metadata.mods:MODS',
         'nl_mods=moai.metadata.mods:NL_MODS',
         'didl=moai.metadata.didl:DIDL',
         'nl_didl=moai.metadata.dare_didl:DareDIDL'
     ],
    },
    install_requires=[
    'pyoai',
    'WSGIUtils',
    'wsgi_intercept',
    'webob',
    'paste',
    'pasteDeploy',
    'pasteScript',
    'sqlalchemy',
    'wsgi_intercept',
    'future',
    'six'
    ],
    test_suite='moai.test.suite'
)





import sys
import datetime
import time
import logging
import logging.handlers
import six


def get_moai_log():
    log = logging.getLogger('moai')
    log.setLevel(logging.DEBUG)
    handler = logging.handlers.RotatingFileHandler('moai.log')
    handler.setFormatter(logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"))
    log.addHandler(handler)
    return log

def get_duration(starttime):
    h,m,s = time.asctime(
        time.gmtime(time.time() - starttime)).split(' ')[-2].split(':')
    s = int(s)
    m = int(m)
    h = int(h)
    duration = '%s second%s' % (int(s), {1:''}.get(s, 's'))
    if m:
        duration = '%s minute%s, %s' % (int(m), {1:''}.get(m, 's'), duration)
    if h:
        duration = '%s hour%s, %s' % (int(h), {1:''}.get(h, 's'), duration)
    return duration

def check_type(object,
               expected_type,
               unicode_keys=False,
               unicode_values=False,
               recursive=False,
               prefix='',
               suffix=''):

    object_type = type(object)
    if not isinstance(object, expected_type):
        
        raise TypeError(('%s expected "%s", got "%s" %s' % (
            prefix,
            expected_type.__name__,
            object.__class__.__name__,
            suffix)).strip())
    if unicode_keys and object_type is dict:
        check_type(object.keys(),
                   list,
                   unicode_values=True,
                   prefix=prefix,
                   suffix=suffix)
    if unicode_values and object_type is dict:
        check_type(list(object.values()),
                   list,
                   unicode_keys=unicode_keys,
                   unicode_values=True,
                   recursive=recursive,
                   prefix=prefix,
                   suffix=suffix)
    if unicode_values and object_type is list:
        for stuff in object:
            if six.PY2:
                if isinstance(stuff, str):
                    raise TypeError(('%s contains non unicode string "%s" %s' % (
                        prefix,
                        stuff,
                        suffix)).strip())
            if recursive:
                if isinstance(stuff, list):
                    check_type(stuff,
                               list,
                               unicode_keys=unicode_keys,
                               unicode_values=True,
                               recursive=True,
                               prefix=prefix,
                               suffix=suffix)
                if isinstance(stuff, dict):
                    check_type(stuff,
                               dict,
                               unicode_keys=unicode_keys,
                               unicode_values=True,
                               recursive=True,
                               prefix=prefix,
                               suffix=suffix)

class XPath(object):
    def __init__(self, doc, nsmap={}):
        self.doc = doc
        self.nsmap = nsmap

    def string(self, xpath):
        return (self.strings(xpath) or [None])[0]

    def strings(self, xpath):
        result = []
        for stuff in self.doc.xpath(xpath, namespaces=self.nsmap):
            if isinstance(stuff, str):
                result.append(stuff.strip().decode('utf8'))
            elif isinstance(stuff, six.text_type):
                # convert to real unicode object, not lxml proxy
                result.append(six.text_type(stuff.strip()))
            elif hasattr(stuff, 'text'):
                if isinstance(stuff.text, str):
                    v = stuff.text.strip()
                    if six.PY2:
                        v = v.decode('utf8')
                    result.append(v)
                elif isinstance(stuff.text, six.text_type):
                    # convert to real unicode object, not lxml proxy
                    result.append(six.text_type(stuff.text.strip()))
        return result
    
    def number(self, xpath):
        return (self.numbers(xpath) or [None])[0]

    def numbers(self, xpath):
        result = []
        for value in self.strings(xpath):
            try:
                value = int(value)
                result.append(value)
            except:
                try:
                    value = float(value)
                    result.append(value)
                except:
                    raise ValueError('Unknown number format: %s' % value)
        return result

    def boolean(self, xpath):
        return (self.booleans(xpath) or [None])[0]

    def booleans(self, xpath):
        result = []
        for value in self.strings(xpath):
            if value.lower() in ['true', 'yes']:
                result.append(True)
            elif value.lower() in ['false', 'no']:
                result.append(False)
            else:
                raise ValueError('Unknown boolean format: %s' % value)
        return result

    def date(self, xpath):
        return (self.dates(xpath) or [None])[0]

    def dates(self, xpath):
        result = []
        for value in self.strings(xpath):
            if 'T' in value:
                if value.endswith('Z'):
                    value = value[:-1] + ' UTC'
                    fmt = '%Y-%m-%dT%H:%M:%S %Z'
                else:
                    fmt = '%Y-%m-%dT%H:%M:%S'
            elif value.count('-') == 2:
                fmt = '%Y-%m-%d'
            elif value.count('/') == 2:
                fmt = '%Y/%m/%d'
            else:
                fmt = '%Y%m%d'
            try:
                result.append(datetime.datetime.strptime(value, fmt))
            except ValueError:
                raise ValueError('Unknown date format: %s' % value)
        return result

    def tag(self, xpath):
        return (self.tags(xpath) or [None])[0]

    def tags(self, xpath):
        result = []
        for stuff in self.doc.xpath(xpath, namespaces=self.nsmap):
            if hasattr(stuff, 'tag'):
                if '}' in stuff.tag:
                    v = stuff.tag.split('}', 1)[1]
                    if six.PY2:
                        v = v.decode('utf8')
                    result.append(v)
                else:
                    v = stuff.tag
                    if six.PY2:
                        v = v.decode('utf8')
                    result.append(v)
        return result
    
    def __call__(self, xpath):
        result = self.doc.xpath(xpath, namespaces=self.nsmap)
        return result


class ProgressBar(object):
    def __init__(self, stream=sys.stderr, width=80):
        self.out = stream
        self.width = width
        self.out.write('')
        self.oldperc = '0.0'
        self.animstate = 0

    def write(self,line):
        self.out.write('\r%s' % line)
        self.out.flush()

    def tick(self, count, total):
        if total:
            perc = '%0.1f' % (count / (total/100.0))
        else:
            perc = '0.0'
        if perc == self.oldperc and not count == total:
            return
        self.oldperc = perc
        lstot = len(str(total))
        awidth = self.width - 10 - lstot
        arrow = ('=' * int(awidth * (float(perc)/100.0)-1))+'>'
        
        self.write(('%5s%%[%-'+str(awidth)+'s] %'+str(lstot)+'d') % (perc,
                                                                     arrow,
                                                                     count))

    def animate(self, msg):
        anim = ['|', '/', '-', '\\']
        rest = self.width - (len(msg) +2)
        self.write('%s %s%s'% (anim[self.animstate], msg, ' '*rest))
        self.animstate+=1
        if self.animstate == len(anim):
            self.animstate = 0






from zope.interface import Interface, Attribute

class IContentProvider(Interface):
    """Object that provides the content used to build the database
    that is used to serve the actual oai data
    """

    def set_logger(self, log):
        """Set the logger instance for this class
        """

    def update(from_date=None):
        """Harvests new content added since from_date
        returns a list of content_ids that were changed/added,
        this should be called before get_contents is called
        """
        pass

    def count():
        """Returns number of content objects in the repository
        returns None if number is unknown, this should not be
        called before update is called
        """

    def get_content_ids():
        """returns a list/generator of content_ids
        """

    def get_content_by_id():
        """Return content of a specific id
        """

class IContentObject(Interface):

    id = Attribute(u"Id of the content object")
    label = Attribute(u"Name of the object")
    content_type = Attribute(u"Type of the content object")
    when_modified = Attribute(u"Modification date of the object")
    deleted = Attribute(u"Boolean that tells if object is deleted or not")
    sets = Attribute(u"A list of ids from sets that this object belongs to")
    is_set = Attribute(u"Boolean indicating if this object is a set")

    provider = Attribute(u"ContentProvider instance that created this object")

    def update(data, provider):
        """Called by IContentProvider, to fill the object with data
        """
        
    def field_names():
        """Return a list of field names, used in this object
        """

    def get_values(field_name):
        """Return a list of python objects (string/int/etc)
        from a specific field name
        """

    def get_assets():
        """Return a list of python dictionaries, each dictionary contains
        at least the following keys:

        - url               - Url of the asset, this will be used in feeds
        - filename          - The filename of the asset
        - md5sum            - md5 checksum of the asset
        - mimetype          - mimetype of the asset
        - absolute_uri      - file:/// or http:/// uri referencing the file
        - metadata          - dictionary with lists of strings as values holding
                              additional metadata
        """
        

        
class IContentValidator(Interface):

    content_type = Attribute("The type of objects this validator can validate")

    def set_logger(logger_instance):
       """Make the validator use a specific custom logger
       (will probably be set automaticly in __init__)
       """
   
    def validate_object(content_object):
        """Validates an object, return a Boolean to indicate validity,
        Alle warnings and errors, should also be logged with the log object
        that's provided as an argument of the set_logger method
        """
    
class IDatabaseUpdater(Interface):

    flush_limit = Attribute('''
        Flush database after processing n records. 
        Defaults to -1, which only flushes the database
        at the end''')

    def set_database(database):
        """Make the updater use a specific (new) database
        (will probably be set automaticly in __init__)
        """

    def set_content_provider(content_provider):
        """Make the updater use a specific ContentProvider
        (will probably be set automaticly in __init__)
        """

    def set_content_class(self, content_object_class):
        """Sets the class to be used to create the content objects
        from the provider data
        """
    
    def set_logger(logger_instance):
       """Make the updater use a specific custom logger
       (will probably be set automaticly in __init__)
       """

    def update_provider(from_date=None):
        """Iterates through update_provider_iterate in a loop,
        returns a list of updated ids
        """

    def update_provider_iterate(from_date=None):
       """Updates the provider from a specific date,
       yields the ids that where updated
       """
   
    def update_database(validate=True, supress_errors=False):
        """Iterates through update_database_iterate in a loop,
        returns the number of errors that occured (int)
        """
    
    def update_database_iterate(validate=True, supress_errors=False):
        """Update the database with the content_provider
        this will update the content_provider, optionally
        validate the content objects, and add everything
        to the database.
        If supress_errors is True, this method should
        never return an error, instead it should
        yield tuples containing the following values:

        (count, total, provider_id, exception)

        count: current content object number
        total: total number of objects
        provider_id: id used by provider
        exception: if an error occurs, exception should be a
                   moai.ContentError, or a moai.DatabaseError
                   otherwise this value will be None

        After the database update is finished, the flush_update method
        of the database is called. This allows the database to
        implement a batching strategy
        """

class IReadOnlyDatabase(Interface):

    def oai_sets(offset=0, batch_size=20):
        """Used by queries from the OAI server. Format returned should be the
        following:

        [{'id': <string>,
          'name': <string>,
          'description': <string>}]
          
        """
    
    def oai_query(offset=0,
                  batch_size=20,
                  sets=[],
                  not_sets=[],
                  filter_sets=[],
                  from_date=None,
                  until_date=None,
                  identifier=None):
        """Used by queries from the OAI server. Format returned should be the
        following:

        [{'record': <dict similar to get_record() output>,
          'metadata': <dict similar to get_metadata() output>,
          'assets': <dict similar to get_assets() output>}
        ]
        """

        
    def get_record(id):
        """Returns a dictionary of data that is available from the
        object with the specific id. The dictionary should contain at least
        the following data:

        {'id': unicode,
        'when_modified': dateTime,
        'deleted': boolean,
        'sets': list of strings,
        }

        If the id does not exist, None is returned
        """

    def get_metadata(id):
        """Returns a dictionary with additional data.
        Keys are always a string, values are always lists of python
        objects.
        
        If the id does not exist, None is returned
        """

    def get_sets(id):
        """Returns a list of set ids for a specific id,
        """

    def get_set(id):
        """Returns a dictionary of set info containing
        - id
        - name
        - description
        
        If the id does not exist, None is returned
        """

    def get_assets(id):
        """Returns a list of dictionaries describing the assets
        Each dictionary contains the following fields:
        - filename
        - url
        - mimetype
        - md5
        - absolute_uri
        - metadata
        
        Where metadata is a dictionary with additional lists of string values
        """

class IDatabase(IReadOnlyDatabase):

    def flush_update():
        """Called once by the database updater at the end of the update proces
        (depending on the flush_threshold attribute in DatabaseUpdater)
        This allows the database to implement a batching strategy
        """
        
    def remove_content(id):
        """Remove all the content of a given id, returns a boolean to indicate
        if the removal was succesful
        """

    def add_content(id, sets, record_data, meta_data, assets_data):
        """Add content to the database, supplying an id and 3 dictionaries,
        of data. The dictionaries should contain at least the keys that
        are needed for generating the get_record, get_metadata and get_keys
        requests.
        Returns a boolean to indicate if the insertion was succesful
        """

    def add_set(id, name, description=None):
        """Add a set to the database
        Returns a boolean to indicate if the insertion was succesful
        """

    def remove_set(id):
        """Remove set from the database
        Returns a boolean to indicate if the removal was succesful
        """

    def empty_database():
        """Removes all data from the database, but doesn't remove the 
        table structures. Mainly used for testing.
        """

        
class IFeedConfig(Interface):

    id = Attribute(u"Id of the OAI Server instance")
    name = Attribute(u"Name of this OAI Server instance (for identify)")
    url = Attribute(u"Base URL of the OAI Server (for identify)")
    log = Attribute(u"Logger instance that logs activity and errors")
    admins = Attribute(u"List of email addresses that can be contacted, "
                      "for questions about the feed")
    metadata_prefixes = Attribute(
        u"List of metadataPrefixes this server can handle"
        "by default the list has 'oai_dc' included")
                   
    # some filter attributes

    content_type = Attribute(u"Type of content objects being served")
    scope = Attribute(u"Only serve objects with a specific (or lower) scope")
    sets_allowed = Attribute(u"Objects served must have one of these sets")
    sets_disallowed = Attribute(
        u"Objects served must not have one of these sets")
    filter_sets = Attribute(
        u"Objects served must have one of these sets, besides the "
        "conforming to the (dis-)allowed sets")
    sets_deleted = Attribute(
        u"Records in this set will always be served as deleted OAI records "
        "this can be used as an alternative to sets_dissallowed.")
    delay = Attribute(u"number of miliseconds to delay the feed")


    def get_oai_id(internal_id):
        """Rename internal id into oai_id"""

    def get_internal_id(oai_id):
        """Rename oai_id into internal id"""

    def get_setspec_id(internal_set_id):
        """Rename internal set id into a setspec id"""

    def get_internal_set_id(oai_setspec_id):
        """Rename setspec id into  internal set id"""

    def get_asset_path(internal_id, asset):
        """Return an absolute path to an asset given
        an internal id the asset data dict containing
        filename, md5, url and metadata
        """

class IServerRequest(Interface):

    def url():
        """Return the current url
        """

    def redirect(url):
        """Redirect to this url
        """

    def send_file(path):
        """Send the file located at 'path' back to the user
        """

    def query_dict():
        """Return a dictionary with QueryString values of the
        request
        """

    def write(data, mimetype):
        """Write data back to the client
        """

    def send_status(code, msg='', mimetype='text/plain'):
        """Return a status code to the user
        """

class IServer(Interface):

    def add_config(config):
        """Add a ServerConfig to the server
        """

    def get_config(id):
        """Get a ServerConfig by id
        """
    
    def download_asset(req, url, config):
        """Download an asset from a url
        """
    
    def allow_download(url, config):
        """Is user allowed to download this asset (returns bool)
        """

    def is_asset_url(url, config):
        """Is this url pointing to an asset (returns bool)
        """
            
    def handle_request(req):
        """Serve this request this method goes through the following steps:
        1. check if url is valid
        2. try to get ServerConfig for this url
        3. test if this is an asset url, if so check if download is allowed,
           and download asset
        4. if not asset url, get the oai server through the OAIServerFactory
        5. call the handleRequest method on the oai server, and return the result
        """
